#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import unittest

from pydantic_core._pydantic_core import ValidationError

import snowprofile


class BaseTestStability:
    result = {}
    additional_keys = {}
    additional_keys_layer = {}

    def test_init_stab(self):
        """
        Check correct instantiation of the class with minimal data
        """
        st = self.CLASS(**self.additional_keys)
        assert st.id is None

    def test_init_stab_result(self):
        """
        Check initialization with a test result
        """
        st_r = self.CLASS_result(**self.result, **self.additional_keys_layer)
        st = self.CLASS(results = [st_r], **self.additional_keys)
        assert st.id is None

    def test_init_stab_result2(self):
        """
        Check adding a second test result
        """
        st_r = self.CLASS_result(**self.result, **self.additional_keys_layer)
        st_r2 = self.CLASS_result(**self.result2, **self.additional_keys_layer)
        st = self.CLASS(results = [st_r], **self.additional_keys)
        st.results.append(st_r2)
        assert len(st.results) == 2


class TestRBStabilityTest(unittest.TestCase, BaseTestStability):
    CLASS = snowprofile.stability_tests.RBStabilityTest
    CLASS_result = snowprofile.stability_tests.RBStabilityTestResult
    result = {'test_score': 2}
    result2 = {'test_score': 5, 'grain_1': 'PP', 'fracture_character': 'SP', 'release_type': 'WB'}
    additional_keys_layer = {'height': 0.21}


class TestCTStabilityTest(unittest.TestCase, BaseTestStability):
    CLASS = snowprofile.stability_tests.CTStabilityTest
    CLASS_result = snowprofile.stability_tests.CTStabilityTestResult
    result = {'test_score': 21}
    result2 = {'test_score': 29, 'grain_1': 'DF', 'fracture_character': 'BRK'}
    additional_keys_layer = {'height': 0.21}


class TestECTStabilityTest(unittest.TestCase, BaseTestStability):
    CLASS = snowprofile.stability_tests.ECTStabilityTest
    CLASS_result = snowprofile.stability_tests.ECTStabilityTestResult
    result = {'test_score': 21}
    result2 = {'test_score': 29, 'grain_1': 'PP', 'propagation': True}
    additional_keys_layer = {'height': 0.21}


class TestShearFrameStabilityTest(unittest.TestCase, BaseTestStability):
    CLASS = snowprofile.stability_tests.ShearFrameStabilityTest
    CLASS_result = snowprofile.stability_tests.ShearFrameStabilityTestResult
    result = {'force': 4}
    result2 = {'force': 8, 'grain_1': 'PP', 'fracture_character': 'SP'}
    additional_keys_layer = {'height': 0.21}


class TestPSTStabilityTest(unittest.TestCase):
    def test_init_stab(self):
        st = snowprofile.stability_tests.PSTStabilityTest(cut_length=0.12, propagation='End')
        assert st.id is None
        st = snowprofile.stability_tests.PSTStabilityTest(cut_length=0.12, column_length=2., propagation='Arr',
                                                          height=0.23, grain_1 = 'SH')
        assert st.id is None


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import unittest
import datetime

from pydantic_core._pydantic_core import ValidationError
import numpy as np

import snowprofile


class BaseTestProfiles:
    values = [100, 75]
    additional_keys = {}

    def test_init_profile(self):
        """
        Check correct instantiation of the class with minimal data
        """
        sp_pd = self.CLASS(data={'top_height': [1, 2],
                                 'thickness': [1, 1],
                                 self.key: self.values, },
                           **self.additional_keys)
        assert sp_pd.id is None

    def test_edit_values(self):
        sp_pd = self.CLASS(data={'top_height': [1, 2],
                                 'thickness': [1, 1],
                                 self.key: self.values, },
                           **self.additional_keys)
        data = sp_pd.data
        data.loc[data.index[1], 'top_height'] += 1

        # check that at this stage values are unchanged and correctly sorted
        assert (sp_pd.data['top_height'].values == np.array([2, 1])).all()

        # Update with inconsistencies
        try:
            sp_pd.data = data
            assert False, 'Inconsistent data should rise an error'
        except ValueError:
            pass

        # Do a correct update
        data.loc[data.index[1], 'thickness'] += 1
        sp_pd.data = data
        # Check correctly updated
        assert (sp_pd.data['top_height'].values == np.array([2, 2])).all()

    def test_init_profile_fail(self):
        """
        check there is a fail if minimal data is not present
        """
        self.assertRaises(
            ValueError,
            self.CLASS, data={'top_height': [1, 2],
                              self.key: self.values, },
            **self.additional_keys
        )

    def test_init_profile_quality_uncertainty_layer(self):
        """
        Check the possibility to add quality and uncertainty flags
        """
        sp_pd = self.CLASS(data={'top_height': [1, 2],
                                 'thickness': [1, 1],
                                 self.key: self.values,
                                 'quality': ['Good', 'Uncertain'],
                                 'uncertainty': [10, 60], },
                           **self.additional_keys)
        assert sp_pd.id is None
        assert (sp_pd.data['quality'].loc[sp_pd.data['top_height'] == 1] == 'Good').all()
        assert (sp_pd.data['quality'].loc[sp_pd.data['top_height'] == 2] == 'Uncertain').all()

        try:
            sp_pd = self.CLASS(data={'top_height': [1, 2],
                                     'thickness': [1, 1],
                                     self.key: self.values,
                                     'quality': ['Good', 'Bof'], },
                               **self.additional_keys)
            assert False, 'Should prevent from incorrect quality flags'
        except ValueError:
            pass

    def test_init_profile_quality_uncertainty_profile(self):
        sp_pd = self.CLASS(data={'top_height': [1, 2],
                                 'thickness': [1, 1],
                                 self.key: self.values, },
                           quality_of_measurement="Bad",
                           uncertainty_of_measurement=20,
                           **self.additional_keys)
        assert sp_pd.quality_of_measurement == "Bad"
        assert sp_pd.uncertainty_of_measurement == 20

        sp_pd.quality_of_measurement = 'Good'
        sp_pd.uncertainty_of_measurement = 22
        assert sp_pd.quality_of_measurement == "Good"
        assert sp_pd.uncertainty_of_measurement == 22

        try:
            sp_pd.quality_of_measurement = "Bof bof"
            assert False, 'Should raise an error when an incorrect value is proposed.'
        except ValidationError:
            pass

        try:
            sp_pd.uncertainty_of_measurement = -22
            assert False, 'Should raise an error when an incorrect value is proposed (uncertainty should be >0).'
        except ValidationError:
            pass

    def test_init_profile_with_metadata(self):
        sp_pd = self.CLASS(data={'top_height': [1, 2],
                                 'thickness': [1, 1],
                                 self.key: self.values, },
                           comment=f"My beautiful {self.key} profile",
                           id=f"profile_{self.key}_01",
                           name=f"{self.key} 01",
                           **self.additional_keys)
        sp_pd.record_time = datetime.datetime.now()
        sp_pd.name = f"{self.key} 02"
        assert sp_pd.name == f"{self.key} 02"


class TestDensityProfile(unittest.TestCase, BaseTestProfiles):
    CLASS = snowprofile.profiles.DensityProfile
    key = 'density'


class TestLWCProfile(unittest.TestCase, BaseTestProfiles):
    CLASS = snowprofile.profiles.LWCProfile
    key = 'lwc'


class TestSSAProfile(unittest.TestCase, BaseTestProfiles):
    CLASS = snowprofile.profiles.SSAProfile
    key = 'ssa'


class TestHardnessProfile(unittest.TestCase, BaseTestProfiles):
    CLASS = snowprofile.profiles.HardnessProfile
    key = 'hardness'


class TestStrengthProfile(unittest.TestCase, BaseTestProfiles):
    CLASS = snowprofile.profiles.StrengthProfile
    key = 'strength'


class TestImpurity1Profile(unittest.TestCase, BaseTestProfiles):
    CLASS = snowprofile.profiles.ImpurityProfile
    key = 'volume_fraction'
    additional_keys = {'impurity_type': 'Black Carbon'}


class TestImpurity2Profile(unittest.TestCase, BaseTestProfiles):
    CLASS = snowprofile.profiles.ImpurityProfile
    key = 'mass_fraction'
    additional_keys = {'impurity_type': 'Black Carbon'}


class TestOtherProfile(unittest.TestCase, BaseTestProfiles):
    CLASS = snowprofile.profiles.ScalarProfile
    key = 'data'
    additional_keys = {'unit': 'm', 'parameter': 'A random length in m',
                       'method_of_measurement': '?'}


class TestOtherVectorialProfile(unittest.TestCase, BaseTestProfiles):
    CLASS = snowprofile.profiles.VectorialProfile
    key = 'data'
    additional_keys = {'unit': 'm', 'parameter': 'A random length in m', 'rank': 2,
                       'method_of_measurement': '?'}
    values = [[100, 100], [75, 76]]

    def test_edit_values(self):
        # Edit has to be checked because it raises a warning
        pass


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-

import datetime
import re
import sys
import os.path
import configparser
import logging


def _parsematch(d, default=None):
    """
    Private function to deal with defined or undefined groups in
    regex match for the `check_and_convert_date` function.
    """
    if d is None:
        return default
    else:
        return int(d)


def check_date(date) -> datetime.datetime:
    """
    Check an input date string and return a
    `bronx.stdtypes.date.Date` object

    Accepted date formats:

    * YYYY[MM[DD[HH[MM[SS]]]]]
    * YYYY-MM-DD[ HH:MM]
    * YYYY-MM-DDTHH:MM:SS
    * Etc (exact regex in the code).

    :param date: Date to be parsed
    :type date: str
    """

    if isinstance(date, datetime.datetime):
        return date
    elif isinstance(date, str):
        # the regex that parses date format
        f1 = re.match('([0-9]{4})[-]?([0-9]{2})?[-]?([0-9]{2})?[ T]?([0-9]{2})?[ :hH]?([0-9]{2})?[ :mM]?([0-9]{2})?',
                      date)
        if f1 is None:
            raise ValueError(f'Could not parse date {date}')
        return datetime.datetime(
            _parsematch(f1.group(1)),             # Year
            _parsematch(f1.group(2), default=8),  # Month
            _parsematch(f1.group(3), default=1),  # Day
            _parsematch(f1.group(4), default=6),  # Hour
            _parsematch(f1.group(5), default=0),  # Min
            _parsematch(f1.group(6), default=0))  # Sec
    else:
        raise ValueError(f'Date should be a datetime object or a date-parseable string. Cannot parse {date}.')


def get_config():
    """
    Get a ConfigParser object of snowprofile configuration.

    Location of configuration file:

    - On UNIX systems, the config file is located in ~/.config/snowprofile.ini
    - On Windows systems, the config file is directly in user home directory
    """
    config = configparser.ConfigParser()
    if sys.platform in ['linux', 'darwin']:
        path_config = os.path.expanduser('~/.config/snowprofile.ini')
    elif sys.platform in ['win32']:
        path_config = os.path.expanduser('~/snowprofile.ini')
    else:
        return config

    if os.path.isfile(path_config):
        try:
            config.read(path_config)
        except Exception as e:
            logging.error(f'Could not load configuration file {path_config}: {e}')

    return config


def get_default_observer(key='DEFAULT'):
    """
    Parse observer details and return an Observer object
    """
    from snowprofile.classes import Observer, Person
    conf = get_config()
    observer = Observer()

    v = conf.get(key, 'observer_id', fallback=None)
    if v is not None:
        observer.source_id = v

    v = conf.get(key, 'observer_name', fallback=None)
    if v is not None:
        observer.source_name = v

    v = conf.get(key, 'observer_comment', fallback=None)
    if v is not None:
        observer.source_comment = v

    cp_id = conf.get(key, 'contact_person_id', fallback=None)
    cp_na = conf.get(key, 'contact_person_name', fallback=None)
    cp_cc = conf.get(key, 'contact_person_comment', fallback=None)
    if cp_na is not None:
        contact_person = Person(id=cp_id, name=cp_na, comment=cp_cc)
        observer.contact_persons = [contact_person, ]

    return observer



# -*- coding: utf-8 -*-

import logging
import xml.etree.ElementTree as ET
import re

from snowprofile.io._caaml_parse_utils import _parse_str, _parse_numeric, _parse_additional_data, \
    _parse_list, _parse_numeric_list, _search_gml_id, _parse_lat_lon
from snowprofile import _constants


def read_caaml5_xml(filename):
    """
    Parse a CAAML 5 XML-based IACS Snow Profile document and return the associated SnowProfile object.

    Versions up to 5.0.5 are currently supported.

    NB : This routine s not extensively tested as CAAML XML v5 is discontinued. No writter will be associated
    and only a minimal reader is provided for getting old data back...

    :param filename: File path of the CAAML/XML document to parse
    :type filename: str
    :returns: The associated SnowProfile python object (or None in case of errors)
    :rtype: SnowProfile
    """
    # XML parsing
    try:
        tree = ET.parse(filename)
        root = tree.getroot()
    except ET.ParseError as e:
        logging.error(f"Fail to parse CAAML XML-based document {filename}. Error while parsing XML file: {str(e)}")
        return None
    # Identification of CAAML namespace
    if root.tag == "SnowProfile":  # Case no namespace
        ns = None
        nss = ''
    else:
        r = re.match('{(.*)}SnowProfile$', root.tag)
        if r is None:
            logging.error(f"The root element of {filename} is not a SnowProfile element. "
                          "This is not a valid CAAML file.")
            return None
        else:
            ns = r.group(1)
            nss = '{' + ns + '}'
    logging.debug(f"Parsing {filename}. Found CAAML namespace as {nss}")

    # Parsing part by part
    from snowprofile import SnowProfile
    from snowprofile.classes import Time, Observer, Location, Environment, Weather, SurfaceConditions

    # - Time
    time = Time(
        record_time=_parse_str(root, [f'{nss}validTime/{nss}TimeInstant/{nss}timePosition',
                                      f'{nss}validTime/{nss}TimePeriod/{nss}beginPosition']),
        record_period=(
            _parse_str(root, f'{nss}validTime/{nss}TimePeriod/{nss}beginPosition'),
            _parse_str(root, f'{nss}validTime/{nss}TimePeriod/{nss}endPosition')),
        report_time=_parse_str(root, f'{nss}metaDataProperty/{nss}MetaData/{nss}dateTimeReport'))

    # - Observer
    observer = Observer(
        source_id=_search_gml_id(root.find(f'{nss}metaDataProperty/{nss}MetaData/{nss}srcRef/{nss}Operation')),
        source_name=_parse_str(root, f'{nss}metaDataProperty/{nss}MetaData/{nss}srcRef/{nss}Operation/{nss}name'))
    contact_persons_1 = root.findall(f'{nss}metaDataProperty/{nss}MetaData/{nss}srcRef/{nss}Operation/{nss}contactPerson')
    contact_persons_2 = root.findall(f'{nss}metaDataProperty/{nss}MetaData/{nss}srcRef/{nss}Person')
    contact_persons = []
    if contact_persons_1 is not None:
        for p in contact_persons_1:
            contact_persons.append(_parse_contact_person(p, nss=nss, ns=ns))
    if contact_persons_2 is not None:
        for p in contact_persons_2:
            contact_persons.append(_parse_contact_person(p, nss=nss, ns=ns))
    if len(contact_persons) > 0:
        observer.contact_persons = contact_persons

    # - Location
    loc = root.find(f'{nss}locRef/{nss}ObsPoint')
    lat, lon = _parse_lat_lon(loc.find(f'{nss}pointLocation'))
    location = Location(
        id=_search_gml_id(loc),
        name=_parse_str(loc, f'{nss}name'),
        point_type=_parse_str(loc, f'{nss}obsPointSubType'),
        aspect=_parse_numeric(loc, f'{nss}validAspect/{nss}AspectPosition/{nss}position',
                              attribution_table=_constants.aspects),
        elevation=_parse_numeric(loc, f'{nss}validElevation/{nss}ElevationPosition/{nss}position'),
        slope=_parse_numeric(loc, f'{nss}validSlopeAngle/{nss}SlopeAnglePosition/{nss}position'),
        latitude=lat,
        longitude = lon)

    # - Environment
    base = f'{nss}snowProfileResultsOf/{nss}SnowProfileMeasurements'
    environment = Environment(
        bed_surface=_parse_str(root, f'{nss}locRef/{nss}obsPointEnvironment/{nss}bedSurface'),
        bed_surface_comment=_parse_str(root, f'{nss}locRef/{nss}obsPointEnvironment/{nss}bedSurfaceComment'),
        litter_thickness=_parse_numeric(root, f'{nss}locRef/{nss}obsPointEnvironment/{nss}litterThickness'),
        ice_thickness=_parse_numeric(root, f'{nss}locRef/{nss}obsPointEnvironment/{nss}iceThickness'),
        low_vegetation_height=_parse_numeric(root, f'{nss}locRef/{nss}obsPointEnvironment/{nss}lowVegetationHeight'),
        LAI=_parse_numeric(root, f'{nss}locRef/{nss}obsPointEnvironment/{nss}lai'),
        forest_presence=_parse_str(root, f'{nss}locRef/{nss}obsPointEnvironment/{nss}forestPresence'),
        forest_presence_comment=_parse_str(root, f'{nss}locRef/{nss}obsPointEnvironment/{nss}forestComment'),
        sky_view_factor=_parse_numeric(root, f'{nss}locRef/{nss}obsPointEnvironment/{nss}skyViewFactor'),
        tree_height=_parse_numeric(root, f'{nss}locRef/{nss}obsPointEnvironment/{nss}treeHeight'))

    # - Weather
    weather = Weather(
        cloudiness=_parse_str(root, f'{base}/{nss}skyCond'),
        precipitation=_parse_str(root, f'{base}/{nss}precipTI'),
        air_temperature=_parse_numeric(root, f'{base}/{nss}airTempPres'),
        wind_speed=_parse_numeric(
            root, f'{base}/{nss}windSpd',
            attribution_table=_constants.wind_speed),
        wind_direction=_parse_numeric(
            root, f'{base}/{nss}windDir/{nss}AspectPosition/{nss}position',
            attribution_table=_constants.aspects))

    # - Surface Conditions
    surface_conditions = SurfaceConditions(
        surface_roughness=_parse_str(
            root,
            f'{base}/{nss}surfFeatures/{nss}Components/{nss}surfRoughness'),
        surface_features_amplitude=_parse_numeric(
            root,
            f'{base}/{nss}surfFeatures/{nss}Components/{nss}validAmplitude/{nss}AmplitudePosition/{nss}position',
            factor=0.01),  # cm -> m
        surface_features_amplitude_min=_parse_numeric(
            root,
            f'{base}/{nss}surfFeatures/{nss}Components/{nss}validAmplitude/{nss}AmplitudeRange/{nss}beginPosition',
            factor=0.01),  # cm -> m
        surface_features_amplitude_max=_parse_numeric(
            root,
            f'{base}/{nss}surfFeatures/{nss}Components/{nss}validAmplitude/{nss}AmplitudeRange/{nss}endPosition',
            factor=0.01),  # cm -> m
        surface_features_wavelength=_parse_numeric(
            root,
            f'{base}/{nss}surfFeatures/{nss}Components/{nss}validWavelength/{nss}WavelengthPosition/{nss}position',
            factor=1),  # m
        surface_features_wavelength_min=_parse_numeric(
            root,
            f'{base}/{nss}surfFeatures/{nss}Components/{nss}validWavelength/{nss}WavelengthRange/{nss}beginPosition',
            factor=1),  # m
        surface_features_wavelength_max=_parse_numeric(
            root,
            f'{base}/{nss}surfFeatures/{nss}Components/{nss}validWavelength/{nss}WavelengthRange/{nss}endPosition',
            factor=1),  # m
        surface_features_aspect=_parse_numeric(
            root,
            f'{base}/{nss}surfFeatures/{nss}Components/{nss}validAspect/{nss}AspectPosition/{nss}position'),
        penetration_ram=_parse_numeric(
            root,
            f'{base}/{nss}penetrationRam',
            factor=0.01),  # cm -> m
        penetration_foot=_parse_numeric(
            root,
            f'{base}/{nss}penetrationFoot',
            factor=0.01),  # cm -> m
        penetration_ski=_parse_numeric(
            root,
            f'{base}/{nss}penetrationSki',
            factor=0.01))  # cm -> m

    # Creating SnowProfile object
    base = f'{nss}snowProfileResultsOf/{nss}SnowProfileMeasurements/{nss}snowPackCond'

    # Profile depth is not taken by default from the profileDepth element, to be coherent
    # with NiViz.
    profile_depth = _parse_numeric(
        root,
        f'{base}/{nss}hS/{nss}Components/{nss}snowHeight',
        factor=0.01)  # cm -> m
    if profile_depth is None:
        profile_depth = _parse_numeric(
            root,
            f'{nss}snowProfileResultsOf/{nss}SnowProfileMeasurements/{nss}profileDepth',
            factor=0.01)

    sp = SnowProfile(
        id=_search_gml_id(root),
        comment=_parse_str(root, f'{nss}metaDataProperty/{nss}MetaData/{nss}comment'),
        profile_comment=_parse_str(
            root,
            f'{nss}snowProfileResultsOf/{nss}SnowProfileMeasurements/{nss}comment'),
        time=time,
        observer=observer,
        location=location,
        environment=environment,
        weather=weather,
        surface_conditions=surface_conditions,
        application=_parse_str(root, f'{nss}application'),
        application_version=_parse_str(root, f'{nss}applicationVersion'),
        profile_depth=profile_depth,
        profile_swe=_parse_numeric(
            root,
            f'{base}/{nss}hS/{nss}Components/{nss}waterEquivalent',
            factor=1),  # kg/m2 is the only one accepted
        stratigraphy_profile = _parse_stratigraphy(
            root.findall(f'{nss}snowProfileResultsOf/{nss}SnowProfileMeasurements/{nss}stratProfile'),
            nss=nss, profile_depth=profile_depth),
        temperature_profiles = _parse_temperature_profiles(
            root.findall(f'{nss}snowProfileResultsOf/{nss}SnowProfileMeasurements/{nss}tempProfile'),
            nss=nss, profile_depth=profile_depth),
        density_profiles = _parse_density_profiles(
            root.findall(f'{nss}snowProfileResultsOf/{nss}SnowProfileMeasurements/{nss}densityProfile'),
            nss=nss, profile_depth=profile_depth),
        lwc_profiles = _parse_lwc_profiles(
            root.findall(f'{nss}snowProfileResultsOf/{nss}SnowProfileMeasurements/{nss}lwcProfile'),
            nss=nss, profile_depth=profile_depth),
        ssa_profiles = _parse_ssa_profiles(
            root.findall(f'{nss}snowProfileResultsOf/{nss}SnowProfileMeasurements/{nss}specSurfAreaProfile'),
            nss=nss, profile_depth=profile_depth),
        hardness_profiles = _parse_hardness_profiles(
            root.findall(f'{nss}snowProfileResultsOf/{nss}SnowProfileMeasurements/{nss}hardnessProfile'),
            nss=nss, profile_depth=profile_depth),
        stability_tests = _parse_stability_tests(
            root.find(f'{nss}snowProfileResultsOf/{nss}SnowProfileMeasurements/{nss}stbTests'),
            nss=nss, profile_depth=profile_depth))

    return sp


def _parse_contact_person(p, nss='', ns=None):
    if p is None:
        return None
    from snowprofile.classes import Person
    return Person(
        id=_search_gml_id(p),
        name=_parse_str(p, f'{nss}name'),
        comment=_parse_str(p, f'{nss}metaData/{nss}comment'),
        additional_data=_parse_additional_data(p.find(f'{nss}customData')))


def _parse_generic_profile(elements, definitions, nss='', min_columns=[]):
    # Eventully get the height to invert depth and height !!
    if elements is None or len(elements) == 0:
        return None
    results = {}
    for key in definitions:
        results[key] = []

    for e in elements:
        for key, value in definitions.items():
            factor = value['numeric_factor'] if 'numeric_factor' in value else 1
            attribute = value['attribute'] if 'attribute' in value else None
            attribution_table = value['attribution_table'] if 'attribution_table' in value else None

            if 'type' in value and value['type'] == 'numeric':
                r = _parse_numeric(e, value['path'], factor=factor, attribute=attribute,
                                   attribution_table=attribution_table)
                if 'adapt_total_depth' in value:
                    r = value['adapt_total_depth'] - r
            elif 'type' in value and value['type'] == 'numeric_list':
                r = _parse_numeric_list(e, value['path'], factor=factor, attribute=attribute)
            else:
                r = _parse_str(e, value['path'], attribute=attribute,
                               attribution_table=attribution_table)
            results[key].append(r)

    # Get rid of columns full of None
    results = {key: value for key, value in results.items() if key in min_columns or set(value) != set([None])}

    return results


def _parse_stratigraphy(elements, nss='', profile_depth=0):
    if elements is None or len(elements) == 0:
        return None

    # Metadata key
    mdk = f'{nss}stratMetaData'

    r = []

    for elem in elements:
        # Get the profile depth
        profile_depth_local = _parse_numeric(elem, f'{mdk}/{nss}hS/{nss}Components/{nss}height',
                                             factor=0.01)  # cm -> m
        if profile_depth_local is not None:
            _profile_depth = profile_depth_local
        else:
            _profile_depth = profile_depth if profile_depth is not None else 0

        data = _parse_generic_profile(
            elem.findall(f'{nss}Layer'),
            {'top_height': {'path': f'{nss}depthTop', 'type': 'numeric',
                            'numeric_factor': 0.01, 'adapt_total_depth': _profile_depth},
             'thickness': {'path': f'{nss}thickness', 'type': 'numeric',
                           'numeric_factor': 0.01},  # cm -> m
             'grain_1': {'path': f'{nss}grainFormPrimary', 'type': 'str'},
             'grain_2': {'path': f'{nss}grainFormSecondary', 'type': 'str'},
             'grain_size': {'path': f'{nss}grainSize/{nss}Components/{nss}avg', 'type': 'numeric',
                            'numeric_factor': 0.001,  # mm -> m
                            'attribution_table': _constants.grain_sizes},
             'grain_size_max': {'path': f'{nss}grainSize/{nss}Components/{nss}avgMax', 'type': 'numeric',
                                'numeric_factor': 0.001,
                                'attribution_table': _constants.grain_sizes},
             'hardness': {'path': f'{nss}hardness', 'type': 'str'},
             'wetness': {'path': f'{nss}lwc', 'type': 'str'},
             'loc': {'path': f'{nss}layerOfConcern', 'type': 'str'},
             'comment': {'path': f'{nss}metaData/{nss}comment', 'type': 'str'},
             'formation_time': {'path': f'{nss}validFormationTime/{nss}TimeInstant/{nss}timePosition', 'type': 'str'},
             'formation_period_begin': {'path': f'{nss}validFormationTime/{nss}TimePeriod/{nss}beginPosition',
                                        'type': 'str'},
             'formation_period_end': {'path': f'{nss}validFormationTime/{nss}TimePeriod/{nss}endPosition',
                                      'type': 'str'}},
            min_columns=['grain_1', 'grain_size', 'hardness', 'grain_2', 'wetness'],
            nss=nss)

        from snowprofile.profiles import Stratigraphy
        s = Stratigraphy(
            id=_search_gml_id(elem),
            name = _parse_str(elem, path='.', attribute='name'),
            related_profiles = _parse_list(elem, '.', attribute='relatedProfiles'),
            comment = _parse_str(elem, f'{mdk}/{nss}comment'),
            record_time = _parse_str(elem, f'{mdk}/{nss}recordTime/{nss}TimeInstant/{nss}timePosition'),
            record_period = (
                _parse_str(elem, f'{mdk}/{nss}recordTime/{nss}TimePeriod/{nss}beginPosition'),
                _parse_str(elem, f'{mdk}/{nss}recordTime/{nss}TimePeriod/{nss}endPosition')),
            profile_depth = profile_depth_local,
            profile_swe = _parse_numeric(elem, f'{mdk}/{nss}hS/{nss}Components/{nss}waterEquivalent'),
            additional_data = _parse_additional_data(elem.find(f'{nss}customData')),
            data = data)
        r.append(s)

    return r[0] if len(r) > 0 else None


def _parse_temperature_profiles(elements, nss='', profile_depth=0):
    if elements is None or len(elements) == 0:
        return []

    # Metadata key
    mdk = f'{nss}MetaData'

    r = []

    for elem in elements:
        # Metadata

        # Get the profile depth
        profile_depth_local = _parse_numeric(elem, f'{mdk}/{nss}hS/{nss}Components/{nss}height',
                                             factor=0.01)  # cm -> m
        if profile_depth_local is not None:
            _profile_depth = profile_depth_local
        else:
            _profile_depth = profile_depth if profile_depth is not None else 0

        data = _parse_generic_profile(
            elem.findall(f'{nss}Obs'),
            {'height': {'path': f'{nss}depth', 'type': 'numeric',
                        'numeric_factor': 0.01, 'adapt_total_depth': _profile_depth},
             'temperature': {'path': f'{nss}snowTemp', 'type': 'numeric'},
             'uncertainty': {'path': f'{nss}uncertaintyOfMeas', 'type': 'str'},
             'quality': {'path': f'{nss}qualityOfMeas', 'type': 'str'}},
            nss=nss)

        from snowprofile.profiles import TemperatureProfile
        s = TemperatureProfile(
            id=_search_gml_id(elem),
            profile_nr = _parse_numeric(elem, path=f'{nss}profileNr'),
            name = _parse_str(elem, path='.', attribute='name'),
            related_profiles = _parse_list(elem, '.', attribute='relatedProfiles'),
            comment = _parse_str(elem, f'{mdk}/{nss}comment'),
            method_of_measurement = _parse_str(elem, f'{mdk}/{nss}methodOfMeas'),
            quality_of_measurement = _parse_str(elem, f'{mdk}/{nss}qualityOfMeas'),
            uncertainty_of_measurement = _parse_numeric(elem, f'{nss}tempMetaData/{nss}uncertaintyOfMeas'),
            record_time = _parse_str(elem, f'{mdk}/{nss}recordTime/{nss}TimeInstant/{nss}timePosition'),
            record_period = (
                _parse_str(elem, f'{mdk}/{nss}recordTime/{nss}TimePeriod/{nss}beginPosition'),
                _parse_str(elem, f'{mdk}/{nss}recordTime/{nss}TimePeriod/{nss}endPosition')),
            profile_depth = profile_depth_local,
            profile_swe = _parse_numeric(elem, f'{mdk}/{nss}hS/{nss}Components/{nss}waterEquivalent'),
            additional_data = _parse_additional_data(elem.find(f'{nss}customData')),
            data = data)
        r.append(s)

    return r


def _parse_density_profiles(elements, nss='', profile_depth=0):
    if elements is None or len(elements) == 0:
        return []

    # Metadata key
    mdk = f'{nss}densityMetaData'

    r = []

    for elem in elements:
        # Metadata

        # Get the profile depth
        profile_depth_local = _parse_numeric(elem, f'{mdk}/{nss}hS/{nss}Components/{nss}height',
                                             factor=0.01)  # cm -> m
        if profile_depth_local is not None:
            _profile_depth = profile_depth_local
        else:
            _profile_depth = profile_depth if profile_depth is not None else 0

        data = _parse_generic_profile(
            elem.findall(f'{nss}Layer'),
            {'top_height': {'path': f'{nss}depthTop', 'type': 'numeric',
                            'numeric_factor': 0.01, 'adapt_total_depth': _profile_depth},
             'thickness': {'path': f'{nss}thickness', 'type': 'numeric',
                           'numeric_factor': 0.01},  # cm -> m
             'density': {'path': f'{nss}density', 'type': 'numeric'},
             'uncertainty': {'path': f'{nss}density', 'attribute': 'uncertainty', 'type': 'str'},
             'quality': {'path': f'{nss}density', 'attribute': 'quality', 'type': 'str'}},
            nss=nss)

        from snowprofile.profiles import DensityProfile
        s = DensityProfile(
            id=_search_gml_id(elem),
            profile_nr = _parse_numeric(elem, path=f'{nss}profileNr'),
            name = _parse_str(elem, path='.', attribute='name'),
            related_profiles = _parse_list(elem, '.', attribute='relatedProfiles'),
            comment = _parse_str(elem, f'{mdk}/{nss}comment'),
            method_of_measurement = _parse_str(elem, f'{mdk}/{nss}methodOfMeas'),
            quality_of_measurement = _parse_str(elem, f'{mdk}/{nss}qualityOfMeas'),
            uncertainty_of_measurement = _parse_numeric(elem, f'{mdk}/{nss}uncertaintyOfMeas'),
            probed_volume = _parse_numeric(elem, f'{mdk}/{nss}probeVolume', factor=1e-6),
            probed_diameter = _parse_numeric(elem, f'{mdk}/{nss}probeDiameter', factor=0.01),
            probed_length = _parse_numeric(elem, f'{mdk}/{nss}probeLength', factor=0.01),
            probed_thickness = _parse_numeric(elem, f'{mdk}/{nss}probedThickness', factor=0.01),
            record_time = _parse_str(elem, f'{mdk}/{nss}recordTime/{nss}TimeInstant/{nss}timePosition'),
            record_period = (
                _parse_str(elem, f'{mdk}/{nss}recordTime/{nss}TimePeriod/{nss}beginPosition'),
                _parse_str(elem, f'{mdk}/{nss}recordTime/{nss}TimePeriod/{nss}endPosition')),
            profile_depth = profile_depth_local,
            profile_swe = _parse_numeric(elem, f'{mdk}/{nss}hS/{nss}Components/{nss}waterEquivalent'),
            additional_data = _parse_additional_data(elem.find(f'{nss}customData')),
            data = data)
        r.append(s)

    return r


def _parse_lwc_profiles(elements, nss='', profile_depth=0):
    if elements is None or len(elements) == 0:
        return []

    # Metadata key
    mdk = f'{nss}lwcMetaData'

    r = []

    for elem in elements:
        # Metadata

        # Get the profile depth
        profile_depth_local = _parse_numeric(elem, f'{mdk}/{nss}hS/{nss}Components/{nss}height',
                                             factor=0.01)  # cm -> m
        if profile_depth_local is not None:
            _profile_depth = profile_depth_local
        else:
            _profile_depth = profile_depth if profile_depth is not None else 0

        data = _parse_generic_profile(
            elem.findall(f'{nss}Layer'),
            {'top_height': {'path': f'{nss}depthTop', 'type': 'numeric',
                            'numeric_factor': 0.01, 'adapt_total_depth': _profile_depth},
             'thickness': {'path': f'{nss}thickness', 'type': 'numeric',
                           'numeric_factor': 0.01},  # cm -> m
             'lwc': {'path': f'{nss}lwc', 'type': 'numeric', 'attribution_table': _constants.attribution_lwc},
             'uncertainty': {'path': f'{nss}lwc', 'attribute': 'uncertainty', 'type': 'str'},
             'quality': {'path': f'{nss}lwc', 'attribute': 'quality', 'type': 'str'}},
            nss=nss)

        from snowprofile.profiles import LWCProfile
        s = LWCProfile(
            id=_search_gml_id(elem),
            profile_nr = _parse_numeric(elem, path=f'{nss}profileNr'),
            name = _parse_str(elem, path='.', attribute='name'),
            related_profiles = _parse_list(elem, '.', attribute='relatedProfiles'),
            comment = _parse_str(elem, f'{mdk}/{nss}comment'),
            method_of_measurement = _parse_str(elem, f'{mdk}/{nss}methodOfMeas'),
            quality_of_measurement = _parse_str(elem, f'{mdk}/{nss}qualityOfMeas'),
            uncertainty_of_measurement = _parse_numeric(elem, f'{mdk}/{nss}uncertaintyOfMeas'),
            probed_thickness = _parse_numeric(elem, f'{mdk}/{nss}probedThickness', factor=0.01),
            record_time = _parse_str(elem, f'{mdk}/{nss}recordTime/{nss}TimeInstant/{nss}timePosition'),
            record_period = (
                _parse_str(elem, f'{mdk}/{nss}recordTime/{nss}TimePeriod/{nss}beginPosition'),
                _parse_str(elem, f'{mdk}/{nss}recordTime/{nss}TimePeriod/{nss}endPosition')),
            profile_depth = profile_depth_local,
            profile_swe = _parse_numeric(elem, f'{mdk}/{nss}hS/{nss}Components/{nss}waterEquivalent'),
            additional_data = _parse_additional_data(elem.find(f'{nss}customData')),
            data = data)
        r.append(s)

    return r


def _parse_ssa_profiles(elements, nss='', profile_depth=0):
    if elements is None or len(elements) == 0:
        return []

    # Metadata key
    mdk = f'{nss}specSurfAreaMetaData'

    r = []

    for elem in elements:
        # Metadata

        # Get the profile depth
        profile_depth_local = _parse_numeric(elem, f'{mdk}/{nss}hS/{nss}Components/{nss}height',
                                             factor=0.01)  # cm -> m
        if profile_depth_local is not None:
            _profile_depth = profile_depth_local
        else:
            _profile_depth = profile_depth if profile_depth is not None else 0

        metadata = dict(
            id=_search_gml_id(elem),
            profile_nr = _parse_numeric(elem, path=f'{nss}profileNr'),
            name = _parse_str(elem, path='.', attribute='name'),
            related_profiles = _parse_list(elem, '.', attribute='relatedProfiles'),
            comment = _parse_str(elem, f'{mdk}/{nss}comment'),
            method_of_measurement = _parse_str(elem, f'{mdk}/{nss}methodOfMeas'),
            quality_of_measurement = _parse_str(elem, f'{mdk}/{nss}qualityOfMeas'),
            uncertainty_of_measurement = _parse_numeric(elem, f'{mdk}/{nss}uncertaintyOfMeas'),
            probed_thickness = _parse_numeric(elem, f'{mdk}/{nss}probedThickness', factor=0.01),
            record_time = _parse_str(elem, f'{mdk}/{nss}recordTime/{nss}TimeInstant/{nss}timePosition'),
            record_period = (
                _parse_str(elem, f'{mdk}/{nss}recordTime/{nss}TimePeriod/{nss}beginPosition'),
                _parse_str(elem, f'{mdk}/{nss}recordTime/{nss}TimePeriod/{nss}endPosition')),
            profile_depth = profile_depth_local,
            profile_swe = _parse_numeric(elem, f'{mdk}/{nss}hS/{nss}Components/{nss}waterEquivalent'),
            additional_data = _parse_additional_data(elem.find(f'{nss}customData')))

        from snowprofile.profiles import SSAProfile, SSAPointProfile

        # Layer profile of SSA
        data1 = _parse_generic_profile(
            elem.findall(f'{nss}Layer'),
            {'top_height': {'path': f'{nss}depthTop', 'type': 'numeric',
                            'numeric_factor': 0.01, 'adapt_total_depth': _profile_depth},
             'thickness': {'path': f'{nss}thickness', 'type': 'numeric',
                           'numeric_factor': 0.01},  # cm -> m
             'ssa': {'path': f'{nss}specSurfArea', 'type': 'numeric'},
             'uncertainty': {'path': f'{nss}specSurfArea', 'attribute': 'uncertainty', 'type': 'str'},
             'quality': {'path': f'{nss}specSurfArea', 'attribute': 'quality', 'type': 'str'}},
            nss=nss)

        if data1 is not None and 'top_height' in data1 and len(data1['top_height']) > 0:
            s = SSAProfile(
                **metadata,
                data = data1)
            r.append(s)

        # Point profile of SSA
        height = []
        ssa = []
        e = elem.find(f'{nss}Measurements/{nss}tupleList')
        if e is not None:
            t = e.text
            try:
                for x in t.split(' '):
                    if len(x.strip()) == 0:
                        continue
                    _tuple = x.split(',')
                    _height = _profile_depth - float(_tuple[0]) / 100
                    _ssa = float(_tuple[1])
                    height.append(_height)
                    ssa.append(_ssa)
            except IndexError:
                logging.error('Could not parse the SSA Profile. tupleList is not well formatted '
                              '(not two element at least in the tuple List (depth adn SSA).'
                              f' Value: {_tuple}')
            except ValueError:
                logging.error('Could not parse the SSA Profile. tupleList is not well formatted '
                              '(element of the tuple list that is not recognized as a float value).'
                              f' Value: {_tuple}')
        if len(height) > 0:
            s = SSAPointProfile(
                **metadata,
                data = {'height': height, 'ssa': ssa})
            r.append(s)

    return r


def _parse_hardness_profiles(elements, nss='', profile_depth=0):
    if elements is None or len(elements) == 0:
        return []

    # Metadata key
    mdk = f'{nss}hardnessMetaData'

    r = []

    for elem in elements:
        # Metadata

        # Get the profile depth
        profile_depth_local = _parse_numeric(elem, f'{mdk}/{nss}hS/{nss}Components/{nss}height',
                                             factor=0.01)  # cm -> m
        if profile_depth_local is not None:
            _profile_depth = profile_depth_local
        else:
            _profile_depth = profile_depth if profile_depth is not None else 0

        metadata = dict(
            id=_search_gml_id(elem),
            profile_nr = _parse_numeric(elem, path=f'{nss}profileNr'),
            name = _parse_str(elem, path='.', attribute='name'),
            related_profiles = _parse_list(elem, '.', attribute='relatedProfiles'),
            comment = _parse_str(elem, f'{mdk}/{nss}comment'),
            method_of_measurement = _parse_str(elem, f'{mdk}/{nss}methodOfMeas'),
            quality_of_measurement = _parse_str(elem, f'{mdk}/{nss}qualityOfMeas'),
            uncertainty_of_measurement = _parse_numeric(elem, f'{mdk}/{nss}uncertaintyOfMeas'),
            surface_of_indentation = _parse_numeric(elem, f'{mdk}/{nss}surfOfIndentation', factor=0.0001),
            penetration_speed = _parse_numeric(elem, f'{mdk}/{nss}penetrationSpeed', factor=1),
            record_time = _parse_str(elem, f'{mdk}/{nss}recordTime/{nss}TimeInstant/{nss}timePosition'),
            record_period = (
                _parse_str(elem, f'{mdk}/{nss}recordTime/{nss}TimePeriod/{nss}beginPosition'),
                _parse_str(elem, f'{mdk}/{nss}recordTime/{nss}TimePeriod/{nss}endPosition')),
            profile_depth = profile_depth_local,
            profile_swe = _parse_numeric(elem, f'{mdk}/{nss}hS/{nss}Components/{nss}waterEquivalent'),
            additional_data = _parse_additional_data(elem.find(f'{nss}customData')))

        from snowprofile.profiles import HardnessProfile, HardnessPointProfile

        # Ramsonde Profile of Hardness
        data1 = _parse_generic_profile(
            elem.findall(f'{nss}Layer'),
            {'top_height': {'path': f'{nss}depthTop', 'type': 'numeric',
                            'numeric_factor': 0.01, 'adapt_total_depth': _profile_depth},
             'thickness': {'path': f'{nss}thickness', 'type': 'numeric',
                           'numeric_factor': 0.01},  # cm -> m
             'hardness': {'path': f'{nss}hardness', 'type': 'numeric'},
             'weight_hammer': {'path': f'{nss}weightHammer', 'type': 'numeric'},
             'weight_tube': {'path': f'{nss}weightTube', 'type': 'numeric'},
             'n_drops': {'path': f'{nss}nDrops', 'type': 'numeric'},
             'drop_height': {'path': f'{nss}dropHeight', 'type': 'numeric',
                             'numeric_factor': 0.01}},  # cm -> m
            nss=nss)

        if data1 is not None and 'top_height' in data1 and len(data1['top_height']) > 0:
            s = HardnessProfile(
                **metadata,
                data = data1)
            r.append(s)

        # Point profile of Hardness
        height = []
        res = []
        e = elem.find(f'{nss}Measurements/{nss}tupleList')
        if e is not None:
            t = e.text
            try:
                for x in t.split(' '):
                    if len(x.strip()) == 0:
                        continue
                    _tuple = x.split(',')
                    _height = _profile_depth - float(_tuple[0]) / 100
                    _res = float(_tuple[1])
                    height.append(_height)
                    res.append(_res)
            except IndexError:
                logging.error('Could not parse the Hardness Profile. tupleList is not well formatted '
                              '(not two element at least in the tuple List (depth adn SSA).'
                              f' Value: {_tuple}')
            except ValueError:
                logging.error('Could not parse the Hardness Profile. tupleList is not well formatted '
                              '(element of the tuple list that is not recognized as a float value).'
                              f' Value: {_tuple}')
        if len(height) > 0:
            s = HardnessPointProfile(
                **metadata,
                data = {'height': height, 'hardness': res})
            r.append(s)

    return r



def _parse_stability_tests(element, nss='', profile_depth=0):
    if element is None:
        return []

    r = []

    # RB tests
    for elementtest in element.findall(f'{nss}RBlockTest'):
        from snowprofile.stability_tests import RBStabilityTest, RBStabilityTestResult
        _results = []
        for e in elementtest.findall(f'{nss}failedOn'):
            test_score = _parse_str(e, f'{nss}Results/{nss}testScore')
            m = re.match('RB([0-7])', test_score)
            if m is None:
                logging.error(f'Could not parse Rutschblock score {test_score}.')
                continue
            test_score = int(m.group(1))
            _s = RBStabilityTestResult(
                **_parse_generic_stability_test_result_fields(e, nss=nss, profile_depth=profile_depth),
                test_score = test_score,
                release_type = _parse_str(e, f'{nss}Results/{nss}releaseType'),
                fracture_character = _parse_str(e, f'{nss}Results/{nss}fractureCharacter'))
            _results.append(_s)
        s = RBStabilityTest(
            **_parse_generic_stability_test_fields(elementtest, nss=nss, profile_depth=profile_depth),
            results = _results)
        r.append(s)

    # CT tests
    for elementtest in element.findall(f'{nss}ComprTest'):
        from snowprofile.stability_tests import CTStabilityTest, CTStabilityTestResult
        _results = []
        for e in elementtest.findall(f'{nss}failedOn'):
            test_score = _parse_str(e, f'{nss}Results/{nss}testScore', attribution_table=_constants.CT_scores)
            try:
                test_score = int(test_score)
            except ValueError:
                logging.error(f'Could not parse ECT score {test_score}.')
                continue
            _s = CTStabilityTestResult(
                **_parse_generic_stability_test_result_fields(e, nss=nss, profile_depth=profile_depth),
                test_score = test_score,
                fracture_character = _parse_str(e, f'{nss}Results/{nss}fractureCharacter'))
            _results.append(_s)
        s = CTStabilityTest(
            **_parse_generic_stability_test_fields(elementtest, nss=nss, profile_depth=profile_depth),
            results = _results)
        r.append(s)

    # ECT tests
    for elementtest in element.findall(f'{nss}ExtColumnTest'):
        from snowprofile.stability_tests import ECTStabilityTest, ECTStabilityTestResult
        _results = []
        for e in elementtest.findall(f'{nss}failedOn'):
            test_score = _parse_str(e, f'{nss}Results/{nss}testScore')
            if test_score == 'ECTPV':
                test_score = 0
                propagation = True
            else:
                m = re.match('ECT([NP])([0-9]*)', test_score)
                if m is None:
                    logging.error(f'Could not parse Rutschblock score {test_score}.')
                    continue
                test_score = int(m.group(2))
                propagation = m.group(1) == 'P'
            _s = ECTStabilityTestResult(
                **_parse_generic_stability_test_result_fields(e, nss=nss, profile_depth=profile_depth),
                test_score = test_score,
                propagation = propagation)
            _results.append(_s)
        s = ECTStabilityTest(
            **_parse_generic_stability_test_fields(elementtest, nss=nss, profile_depth=profile_depth),
            results = _results)
        r.append(s)

    # PST tests
    for elementtest in element.findall(f'{nss}PropSawTest'):
        from snowprofile.stability_tests import PSTStabilityTest
        e = elementtest.find(f'{nss}failedOn')
        if e is not None:
            s = PSTStabilityTest(
                **_parse_generic_stability_test_fields(elementtest, nss=nss, profile_depth=profile_depth),
                **_parse_generic_stability_test_result_fields(e, nss=nss, profile_depth=profile_depth),
                column_length = _parse_numeric(e, f'{nss}Results/{nss}columnLength', factor=0.01),
                cut_length = _parse_numeric(e, f'{nss}Results/{nss}cutLength', factor=0.01),
                propagation = _parse_str(e, f'{nss}Results/{nss}fracturePropagation'))
            r.append(s)

    # Shear frame tests
    for elementtest in element.findall(f'{nss}ShearFrameTest'):
        from snowprofile.stability_tests import ShearFrameStabilityTest, ShearFrameStabilityTestResult
        _results = []
        for e in elementtest.findall(f'{nss}failedOn'):
            _s = ShearFrameStabilityTestResult(
                **_parse_generic_stability_test_result_fields(e, nss=nss, profile_depth=profile_depth),
                force = _parse_numeric(e, f'{nss}Results/{nss}failureForce'),
                fracture_character = _parse_str(e, f'{nss}Results/{nss}fractureCharacter'))
            _results.append(_s)
        s = ShearFrameStabilityTest(
            **_parse_generic_stability_test_fields(elementtest, nss=nss, profile_depth=profile_depth),
            results = _results)
        r.append(s)

    return r


def _parse_generic_stability_test_fields(elem, nss='', profile_depth=0):
    return dict(
        id=_search_gml_id(elem),
        name = _parse_str(elem, path='.', attribute='name'),
        test_nr = _parse_numeric(elem, path=f'{nss}testNr'),
        comment = _parse_str(elem, f'{nss}metaData/{nss}comment'),
        additional_data = _parse_additional_data(elem.find(f'{nss}customData')))


def _parse_generic_stability_test_result_fields(elem, nss='', profile_depth=0):
    depth = _parse_numeric(elem, f'{nss}Layer/{nss}depthTop', factor = 0.01)
    return dict(
        height = profile_depth - depth if depth is not None else None,
        layer_thickness = _parse_numeric(elem, f'{nss}Layer/{nss}thickness', factor = 0.01),
        grain_1 = _parse_str(elem, f'{nss}Layer/{nss}grainFormPrimary'),
        grain_2 = _parse_str(elem, f'{nss}Layer/{nss}grainFormSecondary'),
        grain_size = _parse_numeric(elem, f'{nss}Layer/{nss}grainSize/{nss}Components/{nss}avg', factor=0.001,
                                    attribution_table=_constants.grain_sizes),
        grain_size_max = _parse_numeric(elem, f'{nss}Layer/{nss}grainSize/{nss}Components/{nss}avgMax', factor=0.001,
                                        attribution_table=_constants.grain_sizes),
        layer_formation_time = _parse_str(elem,
                                          f'{nss}Layer/{nss}validFormationTime/{nss}TimeInstant/{nss}timePosition'),
        layer_formation_period = (
            _parse_str(elem, f'{nss}Layer/{nss}validFormationTime/{nss}TimePeriod/{nss}beginPosition'),
            _parse_str(elem, f'{nss}Layer/{nss}validFormationTime/{nss}TimePeriod/{nss}endPosition')),
        layer_comment = _parse_str(elem, f'{nss}Layer/{nss}metaData/{nss}comment'),
        layer_additional_data = _parse_additional_data(elem.find(f'{nss}Layer/{nss}customData')))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import os
import os.path
import unittest
import tempfile

import matplotlib
matplotlib.use('Agg')

import snowprofile
import snowprofile.plot

_here = os.path.dirname(os.path.realpath(__file__))


class TestIOCAAML6XML(unittest.TestCase):

    def test_plot_default_example(self):
        """
        Test file provided on caaml.org website (SnowProfile_IACS_SLF22950.xml)
        """
        sp = snowprofile.io.read_caaml6_xml(os.path.join(_here, 'resources', 'SnowProfile_IACS_SLF22950.xml'))
        fig_f = snowprofile.plot.plot_full(sp)
        fig_s = snowprofile.plot.plot_simple(sp)
        with tempfile.TemporaryDirectory(prefix='snowprofiletests') as dirname:
            filename = os.path.join(dirname, 'fig.png')
            fig_f.savefig(filename)
            fig_s.savefig(filename)

    def test_plot_example3(self):
        """
        Test Profile 3: a rather complete test snow observation.
        """
        sp = snowprofile.io.read_caaml6_xml(os.path.join(_here, 'resources', 'TestProfile3.caaml'))
        fig = snowprofile.plot.plot_full(sp)
        with tempfile.TemporaryDirectory(prefix='snowprofiletests') as dirname:
            filename = os.path.join(dirname, 'fig.png')
            fig.savefig(filename)


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-

import typing

import pydantic

from snowprofile._base_classes import BaseProfile, BaseProfile2
from snowprofile._constants import QUALITY_FLAGS, GRAIN_SHAPES, MANUAL_WETNESS, MANUAL_HARDNESS, \
    manual_wetness_attribution, manual_hardness_attribution

__all__ = ['Stratigraphy',
           'TemperatureProfile', 'DensityProfile', 'LWCProfile',
           'SSAProfile', 'SSAPointProfile',
           'HardnessProfile', 'HardnessPointProfile',
           'StrengthProfile', 'ImpurityProfile',
           'ScalarProfile', 'VectorialProfile']


class Stratigraphy(BaseProfile):
    """
    Measurements of the snowpack stratigraphy, according to the international snowpack classification,
    Fierz et al. 2009.

    The ``data`` key contains:

    - ``top_height``: Top height of the snow layer (m)
    - ``bottom_height``: Bottom height of the snow layer (m)
    - ``thickness``: Thickness of the snow layer (m)
    - ``grain_1``: Primary snow grain shape
    - ``grain_2``: Secondary snow grain shape
    - ``grain_size``: Snow grain size (m)
    - ``grain_size_max``: Maximum snow grain size (m)
    - ``hardness``: Manual hand hardness (intermediates are allowed, details in Fierz et al., 2009):
            - F: Fist (1)
            - 4F : 4 Fingers (2)
            - 1F : 1 Finger (3)
            - P : Pencil (4)
            - K : Knife (5)
    - ``wetness``: Manual snow wetness (intermediates are allowed, details in Fierz et al., 2009):
            - D : Dry (1)
            - M : Moist (2)
            - W : Wet (3)
            - V : Very wet (4)
            - S : Soaked (5)
    - ``formation_time``: Date of the layer formation
    - ``formation_period_begin``: Date of the begining of the layer formation
    - ``formation_period_end``: Date of the end of the layer formation
    - ``loc``: Layer of concern, possible values are 'no', 'top', 'bottom', 'all', 'true', 'false'
    - ``comment``: Comment on the layer
    - ``uncertainty``: Quantitative uncertainty on the layer (see :ref:`uncertainty`)
    - ``quality``: Quality flag on the layer (see :ref:`uncertainty`)
    """

    _data_config = dict(
        grain_1=dict(type='O',
                     values=GRAIN_SHAPES + [None]),
        grain_2=dict(type='O',
                     values=GRAIN_SHAPES + [None]),
        grain_size=dict(min=0,
                        nan_allowed=True),
        grain_size_max=dict(min=0,
                            optional=True,
                            nan_allowed=True),
        hardness=dict(type='O',
                      values=MANUAL_HARDNESS + [None],
                      translate=manual_hardness_attribution,
                      ),
        wetness=dict(type='O',
                     values=MANUAL_WETNESS + [None],
                     translate=manual_wetness_attribution,
                     ),
        loc=dict(type='O',
                 optional=True,
                 values=['no', 'top', 'bottom', 'all', 'true', 'false', None]),
        uncertainty=dict(optional=True,
                         nan_allowed=True),
        quality=dict(optional=True,
                     type='O',
                     values=QUALITY_FLAGS + [None]),
        comment=dict(optional=True,
                     type='O', ),
        additional_data=dict(optional=True,
                             type='O', ),
        formation_time=dict(optional=True,
                            type='O'),
        formation_period_begin=dict(optional=True,
                                    type='O'),
        formation_period_end=dict(optional=True,
                                  type='O'),
    )


class TemperatureProfile(BaseProfile2):
    """
    Vertical profile of temperature profile.

    The ``data`` key contains:

     - ``height``: Height of the temperature measurement (m)
     - ``temperature``: Temperature value (°C)
     - ``uncertainty``: Quantitative uncertainty of the point measurement (same unit as data).
       See :ref:`uncertainty` for details.
     - ``quality``: Quality flag of the point measurement. See :ref:`uncertainty` for details.

    """
    method_of_measurement: typing.Optional[typing.Literal[
        "Thermometer", "IR thermometer", "IR photography", "other"]] = None
    _data_config = dict(
        _mode='Point',
        temperature=dict(max=0),
        uncertainty=dict(optional=True,
                         nan_allowed=True),
        quality=dict(optional=True,
                     type='O',
                     values=QUALITY_FLAGS + [None]),
    )


class DensityProfile(BaseProfile2):
    """
    Vertical profile of density (kg/m3).

    The ``data`` key contains:

    - ``top_height``: Top height of the measurement (m)
    - ``bottom_height``: Bottom height of the measurement (m)
    - ``thickness``: Thickness (m)
    - ``density``: Density value (kg/m3)
    - ``uncertainty``: Quantitative uncertainty of the point measurement (same unit as data).
      See :ref:`uncertainty` for details.
    - ``quality``: Quality flag of the point measurement. See :ref:`uncertainty` for details.
    """
    method_of_measurement: typing.Optional[typing.Literal[
        "Snow Tube", "Snow Cylinder", "Snow Cutter", "Snow wedge Cutter", "other gravimetric measurement method",
        "Denoth Probe", "SnowPro Probe", "Snow Fork", "other dielectric permittivity method",
        "Tomography", "SMP", "Neutron scattering probe",
        "other"]] = pydantic.Field(
            None,
            description="Measurement method")
    probed_volume: typing.Optional[float] = pydantic.Field(
        None,
        description="Probe volume (m3)")
    probed_diameter: typing.Optional[float] = pydantic.Field(
        None,
        description="Probe diameter (m)")
    probed_length: typing.Optional[float] = pydantic.Field(
        None,
        description="Probe length (m)")
    probed_thickness: typing.Optional[float] = pydantic.Field(
        None,
        description="Probe thickness (vertical dimension, m)")
    _data_config = dict(
        density=dict(min=0, max=917),
        uncertainty=dict(optional=True,
                         nan_allowed=True),
        quality=dict(optional=True,
                     type='O',
                     values=QUALITY_FLAGS + [None]),
    )


class LWCProfile(BaseProfile2):
    """
    Vertical profile of LWC (Liquid water content, % by volume).

    The ``data`` key contains:

    - ``top_height``: Top height of the measurement (m)
    - ``bottom_height``: Bottom height of the measurement (m)
    - ``thickness``: Thickness (m)
    - ``lwc``: Liqud water content (% vol.)
    - ``uncertainty``: Quantitative uncertainty of the measurement (same unit as data).
      See :ref:`uncertainty` for details.
    - ``quality``: Quality flag of the measurement. See :ref:`uncertainty` for details.
    """
    method_of_measurement: typing.Optional[typing.Literal[
        "Denoth Probe", "Snow Fork", "SnowPro Probe", "WISe", "other dielectric permittivity method",
        "MRI", "other"]] = pydantic.Field(
            None,
            description="Measurement method")
    probed_thickness: typing.Optional[float] = pydantic.Field(
        None,
        description="Probe thickness (vertical dimension of measurement, m)")
    _data_config = dict(
        lwc=dict(min=0, max=100),
        uncertainty=dict(optional=True,
                         nan_allowed=True),
        quality=dict(optional=True,
                     type='O',
                     values=QUALITY_FLAGS + [None]),
    )


class _SSAProfile(BaseProfile2):
    """
    Vertical profile of SSA (Specific surface area, m2/kg).

    Base class for SSA, whether the profile is a point profile or a layer profile.
    """
    method_of_measurement: typing.Optional[typing.Literal[
        "Ice Cube", "IRIS", "InfraSnow", "DUFFISSS-1310", "DUFFISSS-1550",
        "HISSGraS", "ASSSAP", "other IR integrating sphere",
        "SWIRcam", "SnowImager", "other NIR method",
        "Tomography", "SMP",
        "other"]] = pydantic.Field(
            None,
            description="Measurement method")
    probed_thickness: typing.Optional[float] = pydantic.Field(
        None,
        description="Probe thickness (vertical dimension of measurement, m)")


class SSAProfile(_SSAProfile):
    """
    Vertical profile of SSA (Specific surface area, m2/kg), measured on layers (volume).

    The ``data`` key contains:

    - ``top_height``: Top height of the measurement (m)
    - ``bottom_height``: Bottom height of the measurement (m)
    - ``thickness``: Thickness (m)
    - ``ssa``: SSA value (m2/kg)
    - ``uncertainty``: Quantitative uncertainty of the measurement (same unit as data).
      See :ref:`uncertainty` for details.
    - ``quality``: Quality flag of the measurement. See :ref:`uncertainty` for details.
    """
    _data_config = dict(
        ssa=dict(min=0),
        uncertainty=dict(optional=True,
                         nan_allowed=True),
        quality=dict(optional=True,
                     type='O',
                     values=QUALITY_FLAGS + [None]),
    )
    type: typing.Literal['SSAProfile'] = 'SSAProfile'


class SSAPointProfile(_SSAProfile):
    """
    Vertical profile of SSA (Specific Surface Area), measured at points (heights).

    The ``data`` key contains:

    - ``height``: Height of the measurement (m)
    - ``ssa``: SSA value (m2/kg)
    - ``uncertainty``: Quantitative uncertainty of the measurement (same unit as data).
      See :ref:`uncertainty` for details.
    - ``quality``: Quality flag of the measurement. See :ref:`uncertainty` for details.
    """
    _data_config = dict(
        _mode='Point',
        ssa=dict(min=0),
        uncertainty=dict(optional=True,
                         nan_allowed=True),
        quality=dict(optional=True,
                     type='O',
                     values=QUALITY_FLAGS + [None]),
    )
    type: typing.Literal['SSAPointProfile'] = 'SSAPointProfile'


class _HardnessProfile(BaseProfile2):
    """
    Vertical profile of hardness (N).

    Base class for both RAMSONDE profile or simple hardness profile from other source.
    """
    method_of_measurement: typing.Optional[typing.Literal[
        "SnowMicroPen", "Ram Sonde", "Push-Pull Gauge",
        "Avatech SP1", "Avatech SP2", "Scope propagation labs",
        "other automatic penetrometer",
        "other"]] = pydantic.Field(
            None,
            description="Measurement method")
    surface_of_indentation: typing.Optional[float] = pydantic.Field(
        None,
        description='Indentation surface of the measurement tool (m2)')
    penetration_speed: typing.Optional[float] = pydantic.Field(
        None,
        description='Penetration speed (m/s)')


class HardnessPointProfile(_HardnessProfile):
    """
    Vertical profile of hardness.

    The ``data`` key contains:

    - ``height``: Height of the measurement (m)
    - ``hardness``: Hardness value (N)
    - ``uncertainty``: Quantitative uncertainty of the measurement (same unit as data).
      See :ref:`uncertainty` for details.
    - ``quality``: Quality flag of the measurement. See :ref:`uncertainty` for details.
    """
    _data_config = dict(
        _mode='Point',
        hardness=dict(min=0),
        uncertainty=dict(optional=True,
                         nan_allowed=True),
        quality=dict(optional=True,
                     type='O',
                     values=QUALITY_FLAGS + [None]),
    )
    type: typing.Literal['HardnessPointProfile'] = 'HardnessPointProfile'


class HardnessProfile(_HardnessProfile):
    """
    Vertical profile of hardness profile, measured on snow layers (e.g. for RamSonde measurements).

    The ``data`` key contains:

    - ``top_height``: Top height of the measurement (m)
    - ``bottom_height``: Bottom height of the measurement (m)
    - ``thickness``: Thickness (m)
    - ``hardness`` (N): Hardness value (N)
    - ``weight_hammer`` (kg)
    - ``weight_tube`` (kg)
    - ``n_drops`` (-)
    - ``drop_height`` (m)
    - ``uncertainty``: Quantitative uncertainty of the measurement (same unit as data).
      See :ref:`uncertainty` for details.
    - ``quality``: Quality flag of the measurement. See :ref:`uncertainty` for details.
    """
    _data_config = dict(
        hardness=dict(min=0),
        weight_hammer=dict(min=0, optional=True),
        weight_tube=dict(min=0, optional=True),
        n_drops=dict(type=int, min=0, optional=True),
        drop_height=dict(min=0, optional=True),
        uncertainty=dict(optional=True,
                         nan_allowed=True),
        quality=dict(optional=True,
                     type='O',
                     values=QUALITY_FLAGS + [None]),
    )
    # TODO: Computation of hardness from other data ?
    type: typing.Literal['HardnessProfile'] = 'HardnessProfile'


class StrengthProfile(BaseProfile2):
    """
    Vertical profile of strength (N).

    The ``data`` key contains:

    - ``top_height``: Top height of the measurement (m)
    - ``bottom_height``: Bottom height of the measurement (m)
    - ``thickness``: Thickness (m)
    - ``strength``: Strength value (N)
    - ``fracture_character`` (optional):
        - SP: Sudden planar
        - SC: Sudden collapse
        - SDN: Sudden (both)
        - RP: Resistant planar
        - PC: Progressive compression
        - RES: Resistant (both)
        - BRK or B: Break
        - X: Other/Unknown
    - ``uncertainty``: Quantitative uncertainty of the measurement (same unit as data).
      See :ref:`uncertainty` for details.
    - ``quality``: Quality flag of the measurement. See :ref:`uncertainty` for details.
    """
    method_of_measurement: typing.Optional[typing.Literal[
        "Shear Frame",
        "other"]] = pydantic.Field(
            None,
            description="Measurement method")
    strength_type: typing.Optional[typing.Literal[
        "compressive", "tensile", "shear", "mixed", "other"]] = pydantic.Field(
            None,
            description="Shear direction: compressive, tensile, shear, other or mixed")
    probed_area: typing.Optional[float] = pydantic.Field(
        None,
        description="Probed area (m2)")
    _data_config = dict(
        strength=dict(min=0),
        fracture_character=dict(optional=True,
                                nan_allowed=True,
                                values=["SDN", "SP",
                                        "SC", "RES",
                                        "PC", "RP",
                                        "BRK", "B",
                                        "X", None],
                                type='O'),
        uncertainty=dict(optional=True,
                         nan_allowed=True),
        quality=dict(optional=True,
                     type='O',
                     values=QUALITY_FLAGS + [None]),
    )


class ImpurityProfile(BaseProfile2):
    """
    Vertical profile of impurities (mass or volume fraction).

    The ``data`` key contains:

    - ``top_height``: Top height of the measurement (m)
    - ``bottom_height``: Bottom height of the measurement (m)
    - ``thickness``: Thickness (m)
    - ``mass_fraction``: Mass fraction of impurity (%)
    - ``volume_fraction``: Volume fration of impurity (%)
    - ``uncertainty``: Quantitative uncertainty of the measurement (same unit as data).
      See :ref:`uncertainty` for details.
    - ``quality``: Quality flag of the measurement. See :ref:`uncertainty` for details.
    """
    method_of_measurement: typing.Optional[typing.Literal[
        "other"]] = pydantic.Field(
            None,
            description="Measurement method")
    impurity_type: typing.Literal[
        "Black Carbon", "Dust", "Isotopes", "Other"] = pydantic.Field(
            description="Impurity type for the profile")
    probed_volume: typing.Optional[float] = pydantic.Field(
        None,
        description="Probe volume (m3)")
    probed_diameter: typing.Optional[float] = pydantic.Field(
        None,
        description="Probe volume (m)")
    probed_length: typing.Optional[float] = pydantic.Field(
        None,
        description="Probe length (m)")
    probed_thickness: typing.Optional[float] = pydantic.Field(
        None,
        description="Probe thickness (vertical dimension, m)")
    _data_config = dict(
        mass_fraction=dict(min=0, max=100,
                           optional=True,
                           nan_allowed=True),
        volume_fraction=dict(min=0, max=100,
                             optional=True,
                             nan_allowed=True),
        uncertainty=dict(optional=True,
                         nan_allowed=True),
        quality=dict(optional=True,
                     type='O',
                     values=QUALITY_FLAGS + [None]),
    )


class ScalarProfile(BaseProfile2):
    """
    Other profile of scalar data (not covered by other Profile class).

    The ``data`` key contains:

    - ``top_height``: Top height of the measurement (m)
    - ``bottom_height``: Bottom height of the measurement (m)
    - ``thickness``: Thickness (m)
    - ``data``: Data value
    - ``uncertainty``: Quantitative uncertainty of the measurement (same unit as data).
      See :ref:`uncertainty` for details.
    - ``quality``: Quality flag of the measurement. See :ref:`uncertainty` for details.
    """
    method_of_measurement: str = pydantic.Field(
        description="Measurement method description")
    unit: str = pydantic.Field(
        description="Unit (SI unit please)")
    parameter: str = pydantic.Field(
        description="Measured parameter")
    _data_config = dict(
        data=dict(),
        uncertainty=dict(optional=True,
                         nan_allowed=True),
        quality=dict(optional=True,
                     type='O',
                     values=QUALITY_FLAGS + [None]),
    )


class VectorialProfile(ScalarProfile):
    """
    Other profile of vectorial data (data inherently multi-dimensional, with same unit and type).

    The ``data`` key is a pandas.DataFrame with key data containing a python list of values.
    """
    rank: int = pydantic.Field(
        gt = 1,
        description="Length of the vector (>1, otherwise use OtherScalarProfile)")
    _data_config = dict(
        data=dict(type='O'),
        uncertainty=dict(optional=True,
                         nan_allowed=True),
        quality=dict(optional=True,
                     type='O',
                     values=QUALITY_FLAGS + [None]),
    )



# -*- coding: utf-8 -*-

import json
import logging


def to_dict(snowprofile):
    """
    Dump of a SnowProfile object into a JSON-serializable object
    (combination of dict, list and str).

    :param snowprofile: The SnowProfile object to dump
    :type snowprofile: `:py:class:snowprofile.SnowProfile` object
    :returns: JSON-serializable object (dict)
    """
    return snowprofile.model_dump()


def to_json(snowprofile, **kwargs) -> str:
    """
    Dump of a SnowProfile object into a JSON-encoded string

    :param snowprofile: The SnowProfile object to dump
    :type snowprofile: `:py:class:snowprofile.SnowProfile`
    :param kwargs: Arguments to be passed to the json.dumps function (standard library)
    :returns: JSON-serialized string
    :rtype: str
    """
    return json.dumps(to_dict(snowprofile), **kwargs)


def from_json(json):
    """
    Create a SnowProfile object from a JSON-encoded string.

    :param json: The JSON-encoded string
    :type json: str
    :returns: The corresponding SnowProfile object
    :rtype: `:py:class:snowprofile.SnowProfile`
    """
    from snowprofile import SnowProfile
    new = SnowProfile.model_validate_json(json)
    return new


def write_internal_json(snowprofile, filename, **kwargs):
    """
    Write the SnowProfile object into an internal JSON representation

    :param snowprofile: The SnowProfile object
    :type snowprofile: `:py:class:snowprofile.SnowProfile`
    :param filename: The filename/filepath to write
                     (warning: any existing file with the same name will be overwritten with no confirmation).
    :type filename: str or path-like object
    :param kwargs: Arguments to be passed to the json.dump function (standard library)
    :returns: The written filename
    :rtype: str
    """
    with open(filename, 'w') as ff:
        json.dump(to_dict(snowprofile), ff, **kwargs)
    logging.info(f'Written to {filename}')
    return filename


def read_internal_json(filename):
    """
    Read from an internal JSON representation to create a SnowProfile object.

    :param filename: The filename/filepath to read.
    :type filename: str or path-like object
    :param kwargs: Arguments to be passed to the json.load function (standard library)
    :returns: The corresponding SnowProfile object
    :rtype: `:py:class:snowprofile.SnowProfile`
    """
    with open(filename, 'r') as ff:
        j = json.load(ff)
    return from_json(json.dumps(j))



# -*- coding: utf-8 -*-

import matplotlib.pyplot as plt

from snowprofile.plot import plot_utils


def plot_simple(sp,
                temperature_profiles=[0],
                density_profiles=[0],
                lwc_profiles=[0],
                hardness_profiles=[0],
                **kwargs):
    """
    Quickly plot the data of a :py:class:`snowprofile.snowprofile.SnowProfile` object.

    Once you have a SnowProfile object ``sp``, just do:

    .. code-block:: python

       import matplotlib.pyplot as plt
       from snowprofile.plot import plot_simple
       plot_simple(sp)
       plt.show()

    .. figure:: /images/plot_simple.png
       :align: center

       Example of a simple plot with the example of the profile observed on 2024-12-24 at Col de Porte (France).

    :param sp: SnowProfile object to be plotted
    :type sp: SnowProfile object
    :param temperature_profiles: ``'all'`` to plot all temperature profiles, ``None`` not to plot temperature
                                 or list of indices of the profiles to plot.
    :type temperature_profiles: None, ``'all'`` or list of int
    :param density_profiles: ``'all'`` to plot all density profiles, ``None`` not to plot density
                                 or list of indices of the profiles to plot.
    :type density_profiles: None, ``'all'`` or list of int
    :param lwc_profiles: ``'all'`` to plot all lwc profiles, ``None`` not to plot lwc
                                 or list of indices of the profiles to plot.
    :type lwc_profiles: None, ``'all'`` or list of int
    :param hardness_profiles: ``'all'`` to plot all hardness profiles, ``None`` not to plot hardness
                                 or list of indices of the profiles to plot.
    :type hardness_profiles: None, ``'all'`` or list of int
    :returns: Matplotlib figure

    """

    # Matplotlib figure
    fig, (ax1, ax2) = plt.subplots(nrows=1,
                                   ncols=2,
                                   figsize = (10, 8),
                                   sharey=True,
                                   gridspec_kw={'wspace': 0.2, 'bottom': 0.22, 'top': 0.92})

    if sp.stratigraphy_profile is not None and len(sp.stratigraphy_profile.data) > 0:
        plot_utils.plot_strati_profile(ax2, sp.stratigraphy_profile,
                                       ylabel=None)

    ax1s = [ax1]

    def twinax1():
        ax = ax1.twiny()
        ax1s.append(ax)
        return ax

    # Temperature (blue)
    if len(sp.temperature_profiles) > 0 and temperature_profiles is not None:
        plot_utils.plot_point_profile(ax1, sp.temperature_profiles, 'temperature', temperature_profiles, xlabel = 'Temperature (°C)',
                                      color='b')
        ax1.xaxis.label.set_color('b')
        ax1.set_xlim(-20, 0)

    # Density (green)
    if len(sp.density_profiles) > 0 and density_profiles is not None:
        ax = twinax1()
        plot_utils.plot_vline_profile(ax, sp.density_profiles, 'density', density_profiles, xlabel = 'Density (kg/m3)',
                                      color='g')
        ax.xaxis.label.set_color('g')
        ax.set_xlim(0, 500)

    # LWC (red)
    if len(sp.lwc_profiles) > 0 and lwc_profiles is not None:
        ax = twinax1()
        plot_utils.plot_vline_profile(ax, sp.lwc_profiles, 'lwc', lwc_profiles, xlabel = 'Liquid water content (%)',
                                      color='r')
        ax.xaxis.label.set_color('r')
        ax.set_xlim(10, 0)

    # Hardness (brown)
    if len(sp.hardness_profiles) > 0 and hardness_profiles is not None:
        ax = twinax1()
        plot_utils.plot_step_profile(ax, sp.hardness_profiles, 'hardness', hardness_profiles, xlabel = 'Hardness (N)',
                                     color='tab:brown')
        ax.xaxis.label.set_color('tab:brown')
        ax.set_xlim(1200, 0)

    ax1.set_ylabel('Height (m)')

    if len(ax1s) > 2:
        for i, ax in enumerate(ax1s[2:]):
            ax.xaxis.set_ticks_position("bottom")
            ax.xaxis.set_label_position("bottom")
            ax.spines["bottom"].set_position(("axes", -0.1 * (i + 1)))

    return fig



# -*- coding: utf-8 -*-

import typing
import datetime

import pydantic

from snowprofile._constants import cloudiness_attribution, QUALITY_FLAGS
from snowprofile._base_classes import AdditionalData, BaseData, BaseMergeable, \
    datetime_with_tz, datetime_tuple_with_tz, get_dataframe_checker
from snowprofile._utils import get_config

__all__ = ['Person', 'Time', 'Observer', 'Location', 'Weather', 'SurfaceConditions',
           'Environment', 'SolarMask', 'SpectralAlbedo']

conf = get_config()


class Person(pydantic.BaseModel):
    """
    Class to describe a contact person
    """
    model_config = pydantic.ConfigDict(
        validate_assignment=True,
        extra='forbid')

    id: typing.Optional[str] = None
    name: typing.Optional[str] = None
    website: typing.Optional[str] = None
    comment: typing.Optional[str] = None
    additional_data: typing.Optional[AdditionalData] = None


class Time(pydantic.BaseModel, BaseMergeable):
    """
    Class to store the date and time of observation (and additional date/time considerations)

    If left empty, the time zone will be automatically filled and the time zone is assumed to be UTC.
    """
    model_config = pydantic.ConfigDict(
        validate_assignment=True,
        extra='forbid')

    record_time: datetime_with_tz = pydantic.Field(
        datetime.datetime.now(),
        description="Time of the observation or measurement (python datetime object).")
    record_period: datetime_tuple_with_tz = pydantic.Field(
        (None, None),
        description="Time period of the observation "
        "(tuple of two python datetime objects giving the start time and the end time).")
    report_time: typing.Optional[datetime_with_tz] = pydantic.Field(
        None,
        description="Reporting time of the observation (python datetime object).")
    last_edition_time: typing.Optional[datetime_with_tz] = pydantic.Field(
        None,
        description="Last edition time of the observation (python datetime object).")
    comment: typing.Optional[str] = pydantic.Field(
        None,
        description="Comment on the date and time of observation (str)")
    additional_data: typing.Optional[AdditionalData] = pydantic.Field(
        None,
        description="Field to store additional data for CAAML compatibility (customData), do not use.")


class Observer(pydantic.BaseModel, BaseMergeable):
    """
    Class to store information about the observer and about the institution / lab.

    ``source`` refers to the observation institution and ``person`` to the
    observer.

    ``source_id``
      Unique identifier of the observation institution

    ``source_name``
      Name of the observation institution

    ``source_comment``
      Comment on the observation institution

    ``contact_persons``
      The list of contact persons (or observers)

    ``additional_data``
      Field to store additional data for CAAML compatibility (customData), do not use.
    """
    model_config = pydantic.ConfigDict(
        validate_assignment=True,
        extra='forbid')

    source_id: typing.Optional[str] = conf.get('DEFAULT', 'observer_id', fallback=None)
    source_name: typing.Optional[str] = conf.get('DEFAULT', 'observer_name', fallback=None)
    source_website: typing.Optional[str] = None
    source_comment: typing.Optional[str] = conf.get('DEFAULT', 'observer_comment', fallback=None)
    source_additional_data: typing.Optional[AdditionalData] = None

    contact_persons: typing.List[Person] = pydantic.Field([Person(
        name=conf.get('DEFAULT', 'contact_person_name', fallback=None),
        id=conf.get('DEFAULT', 'contact_person_id', fallback=None),
        comment=conf.get('DEFAULT', 'contact_person_comment', fallback=None)), ], min_length=1)


class Location(pydantic.BaseModel, BaseMergeable):
    """
    Class to store information on the measurement location
    (geographical position and details of the observation site).

    The required field is ``name``.

    ``id``
      Unique identifier of the geographical position

    ``name``
      The name of the observation location (str)

    ``latitude``
      Latitude (degrees north)

    ``longitude``
      Longitude (degrees East)

    ``comment``
      Free comment on the location (str)

    ``elevation``
      Point elevation (meters above sea level)

    ``aspect``
      Slope aspect (degrees, int. between 0 and 360)

    ``slope``
      Slope inclination (degrees, int. between 0 and 90)

    ``point_type``
      A point type description (str)

    ``country``
      Country code according to ISO3166

    ``region``
      Region (detail in the country, optional, str)

    ``additonal_data``
      Field to store additional data for CAAML compatibility (customData), do not use.
    """
    model_config = pydantic.ConfigDict(
        validate_assignment=True,
        extra='forbid')

    id: typing.Optional[str] = None
    name: str
    point_type: typing.Optional[str] = None
    aspect: typing.Optional[int] = pydantic.Field(None, ge=0, le=360)
    elevation: typing.Optional[int] = None
    slope: typing.Optional[int] = pydantic.Field(None, ge=0, lt=90)
    latitude: typing.Optional[float] = None
    longitude: typing.Optional[float] = None
    country: typing.Optional[typing.Literal[
        "AD", "AE", "AF", "AG", "AL", "AM", "AO", "AR", "AT", "AU",
        "AZ", "BA", "BB", "BD", "BE", "BF", "BG", "BH", "BI", "BJ",
        "BN", "BO", "BQ", "BR", "BS", "BT", "BW", "BY", "BZ", "CA",
        "CD", "CF", "CG", "CH", "CI", "CL", "CM", "CN", "CO", "CR",
        "CU", "CV", "CY", "CZ", "DE", "DJ", "DK", "DM", "DO", "DZ",
        "EC", "EE", "EG", "ER", "ES", "ET", "FI", "FJ", "FM", "FR",
        "GA", "GB", "GD", "GE", "GH", "GL", "GM", "GN", "GQ", "GR",
        "GT", "GW", "GY", "HN", "HR", "HT", "HU", "ID", "IE", "IL",
        "IN", "IQ", "IR", "IS", "IT", "JM", "JO", "JP", "KE", "KG",
        "KH", "KI", "KM", "KN", "KP", "KR", "KW", "KZ", "LA", "LB",
        "LC", "LI", "LK", "LR", "LS", "LT", "LU", "LV", "LY", "MA",
        "MC", "MD", "ME", "MG", "MH", "MK", "ML", "MM", "MN", "MR",
        "MT", "MU", "MV", "MW", "MX", "MY", "MZ", "NA", "NE", "NG",
        "NI", "NL", "NO", "NP", "NR", "NZ", "OM", "PA", "PE", "PG",
        "PH", "PK", "PL", "PS", "PT", "PW", "PY", "QA", "RO", "RS",
        "RU", "RW", "SA", "SB", "SC", "SD", "SE", "SG", "SH", "SI",
        "SK", "SL", "SM", "SN", "SO", "SR", "SS", "ST", "SV", "SY",
        "SZ", "TD", "TG", "TH", "TJ", "TL", "TM", "TN", "TO", "TR",
        "TT", "TV", "TW", "TZ", "UA", "UG", "UM", "US", "UY", "UZ",
        "VC", "VE", "VN", "VU", "WF", "WS", "YE", "ZA", "ZM", "ZW"]] = None  # ISO 3166
    region: typing.Optional[str] = None
    comment: typing.Optional[str] = None
    additional_data: typing.Optional[AdditionalData] = None

    @pydantic.field_validator('country', mode='before')
    def _preprocess_country(country: typing.Optional[str]) -> typing.Optional[str]:
        """
        Ensure country code is upper case
        """
        if country is not None:
            return country.upper()
        return None


class Weather(pydantic.BaseModel, BaseMergeable):
    """
    Class to store the weather at time of observation.

    ``cloudiness``
     The cloudiness in octas (from 0 to 8) or in the form of METAR code:
      - CLR: clear
      - FEW: few clouds
      - SCT: scattered
      - BKN: broken
      - OVC: overcast
      - X: precipitation

    ``precipitation``
     Precipitation type, in the form of METAR code:
      - **Nil: No precipitation**
      - DZ: Drizzle
      - **RA: Rain**
      - **SN: Snow** (snow flakes)
      - **SG: Snow grains** (very small opaque grains, generally less than 1 mm)
      - IC: Ice crystals
      - PE: Ice pellets
      - GR: Hail (Grèle)
      - GS: Small hail and/or graupel (Grésil, grains below 5mm)
      - UP: Unknown precipitation type
      - **RASN: Rain and snow**
      - FZRA: Freezing rain

      The precipitation type can be preceded by '-' for light intensity or '+' for heavy intensity.
      The qualifier without +/- is moderate intensity.
      For a definition and pictures of the precipitation types, see e.g.
      `the International Cloud Atlas <https://cloudatlas.wmo.int/en/hydrometeors-other-than-clouds-falling.html>`_


    ``air_temperature``
      Temperature of air (°C)

    ``air_humidity``
      Relative humidity (%)

    ``wind_speed``
      Wind speed (m/s)

    ``wind_direction``
      Wind direction (in degree, from 0 to 360)

    ``air_temperature_measurement_height``
      Height of the air temperature measurement and humidity measurement (m)

    ``wind_measurement_height``
      Height of the wind speed and direction measurement (m)

    ``comment``
      Free comment on the weather

    ``additonal_data``
      Field to store additional data for CAAML compatibility (customData), do not use.
    """
    model_config = pydantic.ConfigDict(
        validate_assignment=True,
        extra='forbid')

    cloudiness: typing.Optional[typing.Literal[
        'CLR', 'FEW', 'SCT', 'BKN', 'OVC', 'X']] = None
    precipitation: typing.Optional[typing.Literal[
        "-DZ", "DZ", "+DZ", "-RA", "RA", "+RA", "-SN", "SN", "+SN",
        "-SG", "SG", "+SG", "-IC", "IC", "+IC", "-PE", "PE", "+PE",
        "-GR", "GR", "+GR", "-GS", "GS", "+GS"
        "UP", "Nil", "RASN", "FZRA"]] = None
    air_temperature: typing.Optional[float] = None
    air_humidity: typing.Optional[float] = pydantic.Field(None, ge=0, le=100)
    wind_speed: typing.Optional[float] = pydantic.Field(None, ge=0)
    wind_direction: typing.Optional[int] = pydantic.Field(None, ge=0, le=360)
    air_temperature_measurement_height: typing.Optional[float] = pydantic.Field(None, gt=0)
    wind_measurement_height: typing.Optional[float] = pydantic.Field(None, gt=0)
    comment: typing.Optional[str] = None
    additional_data: typing.Optional[AdditionalData] = None

    @pydantic.field_validator('cloudiness', mode='before')
    def _preprocess_cloudiness(cloudiness: typing.Optional[str | int]) -> typing.Optional[str]:
        """
        Ensure cloudiness is upper case and convert octas to METAR code
        """
        if cloudiness is not None:
            if isinstance(cloudiness, str):
                return cloudiness.upper()
            elif isinstance(cloudiness, int):
                if cloudiness in cloudiness_attribution:
                    return cloudiness_attribution[cloudiness]
            return cloudiness
        return None


class SpectralAlbedo(pydantic.BaseModel, BaseData, BaseMergeable):
    """
    Class to store spectral albedo data

    The data contains:
      - ``min_wavelength`` (nm)
      - ``max_wavelength`` (nm)
      - ``albedo`` (between 0 and 1)

    and optionnally ``uncertainty`` (same unit as data) and/or ``quality`` (see :ref:`uncertainty`).
    """
    model_config = pydantic.ConfigDict(
        validate_assignment=True,
        extra='forbid',
        arbitrary_types_allowed=True)

    comment: typing.Optional[str] = None
    _data_config = dict(
        _mode='Spectral',
        albedo=dict(min=0, max=1),
        uncertainty=dict(optional=True,
                         nan_allowed=True),
        quality=dict(optional=True,
                     type='O',
                     values=QUALITY_FLAGS + [None]),
    )

    def __init__(self, data=None, data_dict=None, **kwargs):
        super().__init__(**kwargs)
        checker = get_dataframe_checker(**self._data_config)
        if data is not None:
            self._data = checker(data)
        elif data_dict is not None:
            self._data = checker(data_dict)
        else:
            raise ValueError('data key is required')


class SolarMask(pydantic.BaseModel, BaseData):
    """
    Class to store solar mask

    The data contains:

    - ``azimuth`` (degrees from north)
    - ``elevation`` (in degrees from horizontal)

    """
    model_config = pydantic.ConfigDict(
        validate_assignment=True,
        extra='forbid',
        arbitrary_types_allowed=True)

    _data_config = dict(
        _mode='None',
        azimuth=dict(min=0, max=360),
        elevation=dict(min=-90, max=90),
    )

    def __init__(self, data=None, data_dict=None, **kwargs):
        super().__init__(**kwargs)
        checker = get_dataframe_checker(**self._data_config)
        if data is not None:
            self._data = checker(data)
        elif data_dict is not None:
            self._data = checker(data_dict)
        else:
            raise ValueError('data key is required')


class SurfaceConditions(pydantic.BaseModel, BaseMergeable):
    """
    Class to describe the snow surface conditions.

    ``surface_roughness``
     Surface roughness according to Fierz et al., 2009:
      - rsm: smooth
      - rwa: wavy (ripples)
      - rcv: concave furrows (ablation hollows, sun cups, penitents, due to melt or sublimation)
      - rcx: conex furrows (rain or melt groves)
      - rrd: random furrows (due to wind erosion)

    ``surface_wind_features``
     Wind features observable at the surface:
      - No observable wind bedforms
      - Snowdrift around obstacles
      - Snow ripples
      - Snow waves
      - Barchan dunes
      - Dunes
      - Loose patches
      - Pits
      - Snow steps
      - Sastrugi
      - mixed
      - other

    ``surface_melt_rain_features``
     Other surface features:
      - Sun cups
      - Penitents
      - Melt or rain furrows
      - other

    ``surface_features_amplitude``
     Amplitude of the surface features (m)

    ``surface_features_amplitude_max``
     Maximum amplitude of the surface features (m)

    ``surface_features_amplitude_min``
     Minimum amplitude of the surface features (m)

    ``surface_features_wavelength``
     Wavelength of the surface features (m)

    ``surface_features_wavelength_max``
     Maximum wavelength of the surface features (m)

    ``surface_features_wavelength_min``
     Minimum wavelength of the surface features (m)

    ``surface_features_aspect``
     Orientation of surface features (degree, from 0 to 360)

    ``comment``
     Free comment on surface conditions

    ``lap_presence``
     Indication of the presence of light absorbing particule at the snow surface. Values among:
      - No LAP
      - Black Carbon
      - Dust
      - Mixed
      - other

    ``surface_temperature``
     Snow surface temperature (°C)

    ``surface_temperature_measurement_method``
     Measurement method for the surface temperature:
      - Thermometer
      - Hemispheric IR
      - IR thermometer
      - other

    ``surface_albedo``
     Snow surface albedo (0 -1)

    ``surface_albedo_comment``
     Free comment on the snow albedo

    ``spectral_albedo``
     Spectral albedo data, see :py:class:`snowprofile.classes.SpectralAlbedo`

    ``spectral_albedo_comment``
     Free comment on the spectral snow albedo

    ``penetration_foot``
     Depth of snowpack penetration by foot (float, m)

    ``penetration_ram``
     Depth of snowpack penetration with the ramsonde (probe alone) (float, m)

    ``penetration_ski``
     Depth of snowpack penetration by ski (float, m)

    ``additonal_data``
     Field to store additional data for CAAML compatibility (customData), do not use.
    """
    model_config = pydantic.ConfigDict(
        validate_assignment=True,
        extra='forbid')

    surface_roughness: typing.Optional[typing.Literal[
        'rsm', 'rwa', 'rcv', 'rcx', 'rrd']] = None
    surface_wind_features: typing.Optional[typing.Literal[
        "No observable wind bedforms",
        "Snowdrift around obstacles",
        "Snow ripples",
        "Snow waves",
        "Barchan dunes",
        "Dunes",
        "Loose patches",
        "Pits",
        "Snow steps",
        "Sastrugi",
        "mixed",
        "other"]] = None
    surface_melt_rain_features: typing.Optional[typing.Literal[
        "Sun cups",
        "Penitents",
        "Melt or rain furrows",
        "other"]] = None
    surface_features_amplitude: typing.Optional[float] = pydantic.Field(None, gt=0)
    surface_features_amplitude_min: typing.Optional[float] = pydantic.Field(None, gt=0)
    surface_features_amplitude_max: typing.Optional[float] = pydantic.Field(None, gt=0)
    surface_features_wavelength: typing.Optional[float] = pydantic.Field(None, gt=0)
    surface_features_wavelength_min: typing.Optional[float] = pydantic.Field(None, gt=0)
    surface_features_wavelength_max: typing.Optional[float] = pydantic.Field(None, gt=0)
    surface_features_aspect: typing.Optional[int] = pydantic.Field(None, ge=0, le=360)
    lap_presence: typing.Optional[typing.Literal[
        "No LAP", "Black Carbon", "Dust",
        "Mixed", "other"]] = None
    surface_temperature: typing.Optional[float] = None
    surface_temperature_measurement_method: typing.Optional[typing.Literal[
        'Thermometer', 'Hemispheric IR', 'IR thermometer', 'other']] = None
    surface_albedo: typing.Optional[float] = None
    surface_albedo_comment: typing.Optional[str] = None
    spectral_albedo: typing.Optional[SpectralAlbedo] = None
    penetration_ram: typing.Optional[float] = pydantic.Field(None, ge=0)
    penetration_foot: typing.Optional[float] = pydantic.Field(None, ge=0)
    penetration_ski: typing.Optional[float] = pydantic.Field(None, ge=0)
    comment: typing.Optional[str] = None
    additional_data: typing.Optional[AdditionalData] = None


class Environment(pydantic.BaseModel, BaseMergeable):
    """
    Description of the site environment, which is independant of the date and time of observation.

    ``solar_mask``
      The solar mask at the observation site. :py:class:`snowprofile.classes.SolarMask` object.

    ``solar_mask_method_of_measurement``
     Measurement method for the solar mask
      - Theodolite
      - Manual measurement
      - From DTM
      - From DSM
      - other

    ``solar_mask_uncertainty``
     Quantitative uncertainty of the solar mask measurement.

    ``solar_mask_quality``
     Qualitative quality of the solar mask measurement

    ``solar_mask_comment``
     Free comment on the solar mask measurement

    ``bed_surface``
     Characterization of the surface below the snowpack:
      - Sea ice
      - Glacier
      - Ice cap
      - Fresh water ice
      - Wetlands
      - Grassland
      - Shrubs
      - Rocks
      - Bare ground
      - Needle litter
      - Broadleaf litter
      - Artificial surface
      - Mixed
      - Other

    ``bed_surface_comment``
      Free comment on the bed surface

    ``litter_thickness``
      Thickness of the litter, if applicable (m)

    ``ice_thickness``
      Thickness of the ice, if applicable (m)

    ``low_vegetation_height``
      Height of the low vegetation, if applicable (m)

    ``LAI``
      Leaf area index, measured at the vegetation peak (summer, m2/m2).

    ``forest_presence``
      Type of forest, if applicable:
        - Open Area
        - Broadleaf forest
        - Needle forest
        - Mixed forest
        - Shrubs
        - Other

    ``forest_presence_comment``
      Free comment to describe the forest

    ``sky_view_factor``
      In case of forest site, the sky view factor (0 - 1)

    ``tree_height``
      In case of forest site, the mean height of trees (m).

    ``solar_mask_additional_data``
      Field to store additional data for CAAML compatibility (customData), do not use.
    """
    model_config = pydantic.ConfigDict(
        validate_assignment=True,
        extra='forbid',
        arbitrary_types_allowed=True)

    solar_mask: typing.Optional[SolarMask] = pydantic.Field(
        None,
        description="The spectral albedo data.")
    solar_mask_method_of_measurement: typing.Optional[typing.Literal[
        "Theodolite", "Manual measurement", "From DTM", "From DSM", "other"]] = None
    solar_mask_uncertainty: typing.Optional[float] = pydantic.Field(None, ge=0)
    solar_mask_quality: typing.Optional[typing.Literal[tuple(QUALITY_FLAGS)]] = None
    solar_mask_comment: typing.Optional[str] = None
    bed_surface: typing.Optional[typing.Literal[
        "Sea ice", "Glacier", "Ice cap", "Fresh water ice",
        "Wetlands", "Grassland", "Shrubs",
        "Rocks", "Bare ground",
        "Needle litter", "Broadleaf litter",
        "Artificial surface", "Mixed", "Other"]] = None
    bed_surface_comment: typing.Optional[str] = None
    litter_thickness: typing.Optional[float] = pydantic.Field(None, ge=0)
    ice_thickness: typing.Optional[float] = pydantic.Field(None, ge=0)
    low_vegetation_height: typing.Optional[float] = pydantic.Field(None, ge=0)
    LAI: typing.Optional[float] = pydantic.Field(None, ge=0)
    forest_presence: typing.Optional[typing.Literal[
        "Open Area", "Broadleaf forest", "Needle forest",
        "Mixed forest", "Shrubs", "Other"]] = pydantic.Field(
            None,)
    forest_presence_comment: typing.Optional[str] = None
    sky_view_factor: typing.Optional[float] = pydantic.Field(None, ge=0, le=1)
    tree_height: typing.Optional[float] = pydantic.Field(None, ge=0)
    solar_mask_additional_data: typing.Optional[AdditionalData] = None



# -*- coding: utf-8 -*-

import pandas as pd

from snowprofile import SnowProfile
from snowprofile.profiles import DensityProfile, TemperatureProfile, HardnessPointProfile, \
    HardnessProfile, LWCProfile, SSAProfile, SSAPointProfile, StrengthProfile, ImpurityProfile, \
    ScalarProfile, VectorialProfile


def get_mapped_values(df, key, mapping):
    if "column" in mapping:
        return df[mapping["column"]] * mapping.get("scale_factor", 1)
    elif "value" in mapping:
        return mapping["value"]


class_mapper = {
    "DensityProfile": dict(classname=DensityProfile, destination = "density_profiles"),
    "TemperatureProfile": dict(classname=TemperatureProfile, destination = "temperature_profiles"),
    "HardnessPointProfile": dict(classname=HardnessPointProfile, destination = "hardness_profiles"),
    "HardnessProfile": dict(classname=HardnessProfile, destination = "hardness_profiles"),
    "LWCProfile": dict(classname=LWCProfile, destination = "lwc_profiles"),
    "SSAProfile": dict(classname=SSAProfile, destination = "ssa_profiles"),
    "SSAPointProfile": dict(classname=SSAPointProfile, destination = "ssa_profiles"),
    "StrengthProfile": dict(classname=StrengthProfile, destination = "strength_profiles"),
    "ImpurityProfile": dict(classname=ImpurityProfile, destination = "impurity_profiles"),
    "ScalarProfile": dict(classname=ScalarProfile, destination = "other_scalar_profiles"),
    "VectorialProfile": dict(classname=VectorialProfile, destination = "other_vectorial_profiles")
}


def read_csv_profile(filename, mapper={}, profile_class=None, metadata={}, **kwargs):
    """
    Read from a CSV file to create a SnowProfile object, containing the specified single profile.

    The profile is as string refering to a classe defined in the snowprofile.profiles module, namely:

     - DensityProfile :py:class:`snowprofile.profiles.DensityProfile`
     - TemperatureProfile :py:class:`snowprofile.profiles.TemperatureProfile`
     - HardnessPointProfile :py:class:`snowprofile.profiles.HardnessPointProfile`
     - HardnessProfile :py:class:`snowprofile.profiles.HardnessProfile`
     - LWCProfile :py:class:`snowprofile.profiles.LWCProfile`
     - SSAProfile :py:class:`snowprofile.profiles.SSAProfile`
     - SSAPointProfile :py:class:`snowprofile.profiles.SSAPointProfile`
     - StrengthProfile :py:class:`snowprofile.profiles.StrengthProfile`
     - ImpurityProfile :py:class:`snowprofile.profiles.ImpurityProfile`
     - ScalarProfile :py:class:`snowprofile.profiles.ScalarProfile`
     - VectorialProfile :py:class:`snowprofile.profiles.VectorialProfile`

    Example code for reading a density profile from a CSV file containing some comments at the
    begining (8 lines to skip), seperated by tabs.

    .. code-block:: python

       from snowprofile.io.profile_csv import read_csv_profile

       path_density = "densitydata_20240420.txt"

       density_mapper = dict(
           top_height = dict(
               column = "Heigth[cm]",
               scale_factor = 0.01),
           thickness = dict(
               value = 0.025
           ),
           density = dict(
               column = "Density[kg/m3]",
               scale_factor = 1)
       )

       sp = read_csv_profile(
           filename=path_density,
           sep="\\t",
           skiprows=8,
           mapper=density_mapper,
           metadata=dict(
               method_of_measurement="Snow Cutter"),
           profile_class="DensityProfile")


    Each key of the mapper dictionary should map to a key in the data argument of the profile class,
    and the corresponding dictionary should provide:

     - column: the name of the column containing the data in the CSV file
     - scale_factor: a factor to multiply the values in the column (default=1)
     - value: a constant value to be used for all the rows of the profile (alternative to column)

    metadata is a dictionary containing additional information to be added to the profile
    (it's mandatory to provide the parameter and unit keys for ScalarProfile and VectorialProfile
    for example, and when it's not mandatory, it can still be useful).

    :param filename: The filename/filepath to read.
    :type filename: str or path-like object
    :param mapper: A dictionary mapping the keys of the profile to the columns of the CSV file.
    :type mapper: dict
    :param profile_class: The class of the profile to be created
    :type profile_class: str
    :param metadata: Metadata to be added to the profile
    :param kwargs: additional arguments to be passed to the pandas.read_csv function (standard library)
    :returns: The corresponding SnowProfile object
    :rtype: `:py:class:snowprofile.SnowProfile`
    """
    # Checks
    if profile_class is None or profile_class not in class_mapper:
        raise ValueError('Should provide a valid profile_class. '
                         f'Got {profile_class} while accepted values are {", ".join(class_mapper.keys())}')

    # Parsing CSV
    df = pd.read_csv(filename, **kwargs)
    data = pd.DataFrame()
    for key, mapping in mapper.items():
        data[key] = get_mapped_values(df, key, mapping)

    # Creating SnowProfile object
    sp = SnowProfile()
    profile = class_mapper[profile_class]["classname"](data = data, **metadata)
    getattr(sp, class_mapper[profile_class]["destination"]).append(profile)

    return sp



# -*- coding: utf-8 -*-

import logging
import os.path

import numpy as np
import matplotlib
import matplotlib.pyplot as plt
from matplotlib.patches import Rectangle

from snowprofile.plot import plot_dictionnaries

__all__ = ['plot_vline_profile', 'plot_step_profile', 'plot_point_profile', 'plot_strati_profile']


def plot_vline_profile(ax, variable_profiles, name_profile, index_profiles,
                       xlabel = 'Variable (unit)', ylabel = 'Height (m)',
                       color = None,
                       **kwargs):
    """
    Function to plot a set of profiles on an axis.

    .. figure:: /images/plot_vline_profile.png
       :align: center

    """
    colors = iter(plt.cm.tab10(np.linspace(0, 1, 10)))

    if index_profiles == 'all':
        index_profiles = range(len(variable_profiles))

    for i in index_profiles:
        data = variable_profiles[i].data_dict

        if name_profile not in data:
            logging.error(f'Data from key {name_profile} not found.')
            continue

        if color is None:
            c = next(colors)
        else:
            c = color

        if 'top_height' in data and 'bottom_height' in data:
            ax.vlines(x = data[name_profile],
                      ymin = data['bottom_height'],
                      ymax = data['top_height'],
                      label = str(i), color = c, **kwargs)
        elif 'height' in data:
            height = data['height']
            ax.plot(data[name_profile], height,
                    label = str(i), ls = ':', marker='o', **kwargs)

    ax.set_xlabel(xlabel)
    ax.grid(ls=':')
    if ylabel is not None:
        ax.set_ylabel(ylabel)


def plot_step_profile(ax, variable_profiles, name_profile, index_profiles,
                      xlabel = 'Variable (unit)', ylabel = 'Height (m)',
                      color = None,
                      **kwargs):
    """
    Function to plot a set of profiles on an axis.

    .. figure:: /images/plot_step_profile.png
       :align: center

    """
    colors = iter(plt.cm.tab10(np.linspace(0, 1, 10)))

    if index_profiles == 'all':
        index_profiles = range(len(variable_profiles))

    for i in index_profiles:
        data = variable_profiles[i].data_dict

        if name_profile not in data:
            logging.error(f'Data from key {name_profile} not found.')
            continue

        if color is None:
            c = next(colors)
        else:
            c = color

        if 'top_height' in data and 'bottom_height' in data:
            ax.step(x = data[name_profile],
                    y = data['top_height'],
                    label = str(i), color = c, **kwargs)
            ax.vlines(x = data[name_profile][-1],
                      ymax = data['top_height'][-1],
                      ymin = data['bottom_height'][-1], color = c, **kwargs)
        elif 'height' in data:
            logging.error('A step plot was not possible as no tickness is associated to the data.')
            height = data['height']
            ax.plot(data[name_profile], height,
                    label = str(i), ls = ':', marker='o', **kwargs)

    ax.set_xlabel(xlabel)
    ax.grid(ls=':')
    if ylabel is not None:
        ax.set_ylabel(ylabel)


def plot_point_profile(ax, variable_profiles, name_profile, index_profiles,
                       xlabel = 'Variable (unit)', ylabel = 'Height (m)',
                       color = None,
                       **kwargs):
    """
    Function to plot a set of profiles on an axis.

    .. figure:: /images/plot_point_profile.png
       :align: center

    """
    colors = iter(plt.cm.tab10(np.linspace(0, 1, 10)))

    if index_profiles == 'all':
        index_profiles = range(len(variable_profiles))

    for i in index_profiles:
        data = variable_profiles[i].data_dict

        if name_profile not in data:
            logging.error(f'Data from key {name_profile} not found.')
            continue

        if color is None:
            c = next(colors)
        else:
            c = color

        if 'top_height' in data and 'thickness' in data:
            height = np.array(data['top_height']) - np.array(data['thickness']) / 2
        elif 'height' in data:
            height = data['height']
        else:
            logging.error('Either height or top_height/thickness should be provided')
            continue
        ax.plot(variable_profiles[i].data_dict[name_profile], height,
                label = str(i), ls = ':', marker='o', color=c, **kwargs)

    ax.set_xlabel(xlabel)
    ax.grid(ls=':')
    if ylabel is not None:
        ax.set_ylabel(ylabel)


def plot_strati_profile(ax, stratigraphy, xlabel = 'Hand harness', ylabel = 'Height (m)',
                        grain_labels=True, use_hardness=True, **kwargs):
    """
    Plot stratigraphy- (layers, colored according to grains, with grain symbol
    and with the hand-hardness)

    .. figure:: /images/plot_strati_profile.png
       :align: center


    :param ax: Matplotlib axis to plot on
    :type ax: Matplotlib axis
    :param stratigraphy: SnowProfile stratigraphy element to plot
    :type stratigraphy: :py:class:`snowprofile.classes.Stratigraphy`
    :param xlabel: The x-axis label
    :type xlabel: str
    :param ylabel: The y-ayis label
    :type ylabel: str
    :param grain_labels: Whether or not to plot the grain labels
    :type grain_labels: bool
    :param use_hardness: Whether or not to use the hand hardness

    """

    _here = os.path.dirname(os.path.realpath(__file__))
    snowiacs = os.path.join(_here, './SnowSymbolsIACS.ttf')
    if not os.path.isfile(snowiacs):
        snowiacs = matplotlib.font_manager.findfont('SnowSymbolsIACS')
    snowsymb = matplotlib.font_manager.FontProperties(fname=snowiacs)

    # get data
    data = stratigraphy.data_dict
    height = data['top_height']
    height.append(data['bottom_height'][-1])

    if use_hardness:
        hardness = data['hardness']
    else:
        hardness = [1] * (len(height) - 1)
    grain1 = data['grain_1']
    grain2 = data['grain_2']

    for i in range(len(grain1)):
        c = plot_dictionnaries.get_grain_color(grain1[i])
        width = plot_dictionnaries.get_hardness_value(hardness[i])  # width of the rectangle
        delta_h = height[i] - height[i + 1]  # height of the rectangle

        ax.add_patch(Rectangle((0., height[i + 1]), width, delta_h, color = c))

        ax.hlines(y=height[i + 1], xmin=0, xmax=width, linewidth=1, color='k')
        ax.hlines(y=height[i], xmin=0, xmax=width, linewidth=1, color='k')

        if grain_labels:
            text = plot_dictionnaries.get_grain_text(grain1[i]) + plot_dictionnaries.get_grain_text(grain2[i])
            ax.text(width / 2, height[i + 1] + delta_h / 2, text,
                    horizontalalignment='center', verticalalignment='center',
                    fontproperties=snowsymb, fontsize=12, color='k', fontweight='bold')

    ax.set_xlim(xmax=6)
    ax.set_xlabel(xlabel)
    ax.grid(ls=':')
    if ylabel is not None:
        ax.set_ylabel(ylabel)
    if use_hardness:
        tick_hardness_str = list(plot_dictionnaries.hardness_str_values.keys())
        tick_hardness_nb = list(plot_dictionnaries.hardness_str_values.values())
        ax.set_xticks(tick_hardness_nb)  # Set the tick positions
        ax.set_xticklabels(tick_hardness_str)  # Set the tick labels



# -*- coding: utf-8 -*-

from snowprofile.snowprofile import SnowProfile
from snowprofile import profiles
from snowprofile import classes
from snowprofile import stability_tests
from snowprofile import io

__all__ = ['SnowProfile', 'profiles', 'classes', 'stability_tests', 'io']



# -*- coding: utf-8 -*-

import logging

import matplotlib.pyplot as plt

from snowprofile.plot import plot_utils


def plot_full(sp,
              index_temperature_profiles = 'all',
              index_density_profiles = 'all', style_density_profiles = 'step',
              index_hardness_profiles = 'all', style_hardness_profiles = 'step',
              index_impurity_profiles = 'all', style_impurity_profiles = 'point',
              index_ssa_profiles = 'all', style_ssa_profiles = 'point',
              index_strength_profiles = 'all', style_strength_profiles = 'point',
              index_lwc_profiles = 'all', style_lwc_profiles = 'step',
              index_scalar_profiles = 'all',
              **kwargs):

    """
    Plot the data (only the vertical property profiles) of a :py:class:`snowprofile.snowprofile.SnowProfile` object.

    Once you have a SnowProfile object ``sp``, just do:

    .. code-block:: python

       import matplotlib.pyplot as plt
       from snowprofile.plot import plot_full
       plot_full(sp)
       plt.show()

    .. figure:: /images/plot_full.png
       :align: center

    **Style** : to be chosen for nearly all variables : may be ``'point'`` to plot with markers or
    ``'step'`` to do a step plot (only possible when thickness is correctly provided).

    **Index**: Except for stratigraphy, several profiles of each variables can be stored in SnowProfile object.
    You can select the profiles to plot by passing a list of indices or use the default value ``'all'`` to
    plot all available profiles.

    :param sp: SnowProfile object to be plotted
    :type sp: SnowProfile object
    :param index_temperature_profiles: index of the temperature profile to be plotted
    :type index_temperature_profiles: list of int

    :type index_hardness_profiles: list of int
    :param style_hardness_profiles: plotting style for hardness profiles
    :type style_hardness_profiles: str
    :type index_impurity_profiles: list of int
    :param style_impurity_profiles: plotting style for impurity profiles
    :type style_impurity_profiles: str
    :param index_ssa_profiles: index of the ssa profile to be plotted
    :type index_ssa_profiles: list of int
    :param style_ssa_profiles: plotting style for SSA profiles
    :type style_ssa_profiles: str
    :param index_strength_profiles: index of the strength profile to be plotted
    :type index_strength_profiles: list of int
    :param style_strength_profiles: plotting style for strength profiles
    :type style_strength_profiles: str
    :param index_lwc_profiles: index of the lwc profile to be plotted
    :type index_lwc_profiles: list of int
    :param style_lwc_profiles: plotting style for LWC profiles
    :type style_lwc_profiles: str
    :type index_scalar_profiles: list of int
    :param index_density_profiles: index of the density profile to be plotted
    :type index_density_profiles: list of int
    :param style_density_profiles: plotting style for density profiles
    :type style_density_profiles: str
    :returns: Matplotlib figure

    """

    to_plot = {
        'Stratigraphy profile': {
            'plot_style': 'profile',
            'key': 'hardness',
            'xlabel': 'Hand Hardness',
            'data': sp.stratigraphy_profile,
            'index': None},
        'Hardness profile': {
            'plot_style': style_hardness_profiles,
            'key': 'hardness',
            'xlabel': 'Hardness (N)',
            'data': sp.hardness_profiles,
            'index': index_hardness_profiles},
        'Temperature profile': {
            'plot_style': 'point',
            'key': 'temperature',
            'xlabel': 'Temperature (°C)',
            'data': sp.temperature_profiles,
            'index': index_temperature_profiles},
        'Density profile': {
            'plot_style': style_density_profiles,
            'key': 'density',
            'xlabel': 'Density (kg/m3)',
            'data': sp.density_profiles,
            'index': index_density_profiles},
        'LWC': {
            'plot_style': style_lwc_profiles,
            'key': 'lwc',
            'xlabel': 'LWC (%)',
            'data': sp.lwc_profiles,
            'index': index_lwc_profiles},
        'Strength profile': {
            'plot_style': style_strength_profiles,
            'key': 'strength',
            'xlabel': 'Strength (N)',
            'data': sp.strength_profiles,
            'index': index_strength_profiles},
        'SSA profile': {
            'plot_style': style_ssa_profiles,
            'key': 'ssa',
            'xlabel': 'SSA (m2/kg)',
            'data': sp.ssa_profiles,
            'index': index_ssa_profiles},
        'Impurity profile': {
            'plot_style': style_impurity_profiles,
            'key': 'mass_fraction',
            'xlabel': 'Impurity mass fraction',
            'data': sp.impurity_profiles,
            'index': index_impurity_profiles},
        'Other scalar profile': {
            'plot_style': 'point',
            'key': 'data',
            'xlabel': 'Other scalar variable',
            'data': sp.other_scalar_profiles,
            'index': index_scalar_profiles}, }

    step_profiles_key_list = ['hardness']

    n_to_plot = 0
    for p, v in to_plot.items():
        if v['data'] is not None and (not isinstance(v['data'], list)
                                      or (len(v['data']) > 0
                                      and v['index'] is not None)):
            v['ok'] = True
            n_to_plot += 1
        else:
            v['ok'] = False

    # Matplotlib figure
    fig, axs = plt.subplots(nrows=(n_to_plot - 1) // 4 + 1,
                            ncols=min(n_to_plot, 4),
                            figsize = (16, 15),
                            sharey=True,
                            gridspec_kw={'wspace': 0.4})
    axes = sum([list(a) for a in axs], [])

    n_subplots = 0
    for p, v in to_plot.items():
        if not v['ok']:
            continue

        common_kwargs = {
            'ylabel': 'Height (m)' if n_subplots % 4 == 0 else None}
        if v['plot_style'] == 'profile':
            plot_utils.plot_strati_profile(
                axes[n_subplots], v['data'], xlabel = v['xlabel'],
                **common_kwargs, **kwargs)
        elif v['plot_style'] == 'point':
            plot_utils.plot_point_profile(
                axes[n_subplots], v['data'], v['key'], v['index'], xlabel = v['xlabel'],
                **common_kwargs, **kwargs)
        elif v['plot_style'] == 'step':
            if v['key'] in step_profiles_key_list:
                plot_utils.plot_step_profile(
                    axes[n_subplots], v['data'], v['key'], v['index'], xlabel = v['xlabel'],
                    **common_kwargs, **kwargs)
            else:
                plot_utils.plot_vline_profile(
                    axes[n_subplots], v['data'], v['key'], v['index'], xlabel = v['xlabel'],
                    **common_kwargs, **kwargs)
        else:
            logging.error(f"Unknown style {v['plot_style']}. Should be either point or step.")
        n_subplots += 1

    return fig



# -*- coding: utf-8 -*-

from snowprofile.io.internal_json import to_dict, to_json, write_internal_json, read_internal_json, from_json
from snowprofile.io.caaml6_xml import read_caaml6_xml, write_caaml6_xml
from snowprofile.io.caaml5_xml import read_caaml5_xml
from snowprofile.io.mf_bdclim import read_mf_bdclim, search_mf_bdclim_dates
from snowprofile.io.profile_csv import read_csv_profile



# -*- coding: utf-8 -*-

import os
import os.path
import unittest

import pandas as pd

import snowprofile


_here = os.path.dirname(os.path.realpath(__file__))


class TestIOCsvProfile(unittest.TestCase):

    def test_read_density_profile(self):
        path_density = os.path.join(_here, "resources", "densitydata_20240420.txt")
        density_mapper = dict(
            top_height = dict(
                column = "Heigth[cm]",
                scale_factor = 0.01),
            thickness = dict(
                value = 0.025
            ),
            density = dict(
                column = "Density[kg/m3]",
                scale_factor = 1)
        )
        sp = snowprofile.io.read_csv_profile(path_density, sep="\t", skiprows=8, mapper=density_mapper,
                                             profile_class="DensityProfile")
        assert(len(sp.density_profiles) == 1)
        expected_density = pd.Series(
            [113.68, 137.12, 177.08, 130.0, 286.24, 273.36, 302.44, 291.92, 309.04, 333.48, 223.04])
        assert sp.density_profiles[0].data["density"].equals(expected_density)

    def test_read_scalar_profile(self):
        path_scalar = os.path.join(_here, "resources", "medianDerivatives.csv")
        scalar_point_mapper = dict(
            top_height = dict(
                column = "corrected height [mm]",
                scale_factor = 0.001),
            thickness = dict(
                value = 0.005
            ),
            data = dict(
                column = "L2012_lambda [1/mm]", scale_factor = 1000
            ),
            uncertainty = dict(
                value = 0.005  # Arbitrary value
            ),
            quality = dict(
                value = "Good"
            )
        )
        sp = snowprofile.io.read_csv_profile(
            path_scalar,
            sep=",",
            skiprows=0,
            mapper=scalar_point_mapper,
            profile_class="ScalarProfile",
            metadata= dict(
                parameter = "L2012_lambda",
                unit = "1/m",
                method_of_measurement = "Unknown"
            )
        )
        assert len(sp.other_scalar_profiles) == 1
        assert sp.other_scalar_profiles[0].data.loc[:10, "data"].equals(
            pd.Series(
                [21042069.4, 311975.5, 238303.5, 1511739.5, 281864.0,
                 569550.1000000001, 726504.5, 1265936.3, 334452.7, 367673.80000000005, 332738.5]))
        assert sp.other_scalar_profiles[0].parameter == "L2012_lambda"
        assert sp.other_scalar_profiles[0].unit == "1/m"



# -*- coding: utf-8 -*-

import typing

import pydantic

from snowprofile._base_classes import AdditionalData, datetime_with_tz, datetime_tuple_with_tz
from snowprofile._constants import GRAIN_SHAPES

__all__ = ['CTStabilityTest', 'CTStabilityTestResult',
           'ECTStabilityTest', 'ECTStabilityTestResult',
           'PSTStabilityTest',
           'RBStabilityTest', 'RBStabilityTestResult',
           'ShearFrameStabilityTest', 'ShearFrameStabilityTestResult']


class _StabilityTest(pydantic.BaseModel):
    """
    Base class for Stability test representation.
    """
    model_config = pydantic.ConfigDict(
        validate_assignment=True,
        extra='forbid')

    id: typing.Optional[str] = None
    name: typing.Optional[str] = pydantic.Field(
        None,
        description="Name/short description of the test")
    related_profiles: typing.List[str] = pydantic.Field(
        [],
        description="id of related profiles")
    comment: typing.Optional[str] = None
    test_nr: typing.Optional[int] = pydantic.Field(
        None, ge=0,
        description="Test number (the lower is the higher priority)")
    name: typing.Optional[str] = pydantic.Field(
        None,
        description="Name/short description of the profile")
    comment: typing.Optional[str] = pydantic.Field(
        None,
        description="A comment associated to the profile")
    additional_data: typing.Optional[AdditionalData] = pydantic.Field(
        None,
        description="Field to store additional data for CAAML compatibility (customData), do not use.")


class _StabilityTestResult(pydantic.BaseModel):
    """
    Base class for stability tests result representation.

    """
    model_config = pydantic.ConfigDict(
        validate_assignment=True,
        extra='forbid')

    height: typing.Optional[float] = pydantic.Field(
        None,
        description="Height of the failed layer (with zero at bottom of the snowpack (m)")
    layer_thickness: typing.Optional[float] = pydantic.Field(
        None,
        description="Thickness of the failed layer (m)")
    grain_1: typing.Optional[typing.Literal[tuple(GRAIN_SHAPES)]] = pydantic.Field(
        None,
        description="Primary grain shape of the failed layer")
    grain_2: typing.Optional[typing.Literal[tuple(GRAIN_SHAPES)]] = pydantic.Field(
        None,
        description="Secondary grain shape of the failed layer")
    grain_size: typing.Optional[float] = pydantic.Field(
        None,
        description="Snow grain size of the failed layer (m)")
    grain_size_max: typing.Optional[float] = pydantic.Field(
        None,
        description="Maximum snow grain size of the failed layer (m)")
    layer_formation_time: typing.Optional[datetime_with_tz] = pydantic.Field(
        None,
        description="The formation time of the failed layer.")
    layer_formation_period: typing.Optional[datetime_tuple_with_tz] = pydantic.Field(
        (None, None),
        description="The formation period (begin, end) of the failed layer.")
    layer_comment: typing.Optional[str] = pydantic.Field(
        None,
        description="Comment associated to layer description (for CAAML compatibility only, do not fill).")
    layer_additional_data: typing.Optional[AdditionalData] = pydantic.Field(
        None,
        description="Field to store additional data for CAAML compatibility (customData), do not use.")


class RBStabilityTestResult(_StabilityTestResult):
    """
    Class for RB (Rutschblock) stability test results.
    """
    model_config = pydantic.ConfigDict(
        validate_assignment=True,
        extra='forbid')

    test_score: int = pydantic.Field(
        description="RB result [1-7], 7 meaning no fracture",
        ge=1, le=7)
    release_type: typing.Optional[typing.Literal[
        'WB', 'MB', 'EB']] = pydantic.Field(
            None,
            description="Release type among:\n\n"
            "- WB: Whole block\n"
            "- MB: Most of the block\n"
            "- EB: Edge of the block")
    fracture_character: typing.Optional[typing.Literal[
        'SDN', 'SP', 'SC', 'RES', 'RP', 'PC', 'BRK',
        'Clean', 'Rough', 'Irregular',
        'Q1', 'Q2', 'Q3']] = pydantic.Field(
            None,
            description="Fracture characteristic among: \n\n"
            "- SP: Sudden planar\n"
        "- SC: Sudden collapse\n"
        "- SDN: Sudden (both)\n"
        "- RP: Resistant planar\n"
        "- PC: Progressive compression\n"
        "- RES: Resistant (both)\n"
        "- BRK or B: Break\n"
        "- Clean\n"
        "- Rough\n"
        "- Irregular\n"
        "- Q1\n"
        "- Q2\n"
        "- Q3")


class RBStabilityTest(_StabilityTest):
    """
    Class for RB (Rutschblock) stability test.
    """
    results: typing.List[RBStabilityTestResult] = pydantic.Field(
        [],
        description="Successive results of a single RB test. No results mean RB7.")
    type: typing.Literal['Rutschblock'] = 'Rutschblock'


class CTStabilityTestResult(_StabilityTestResult):
    """
    Class for CT (Comprerssion test) stability test results.
    """
    model_config = pydantic.ConfigDict(
        validate_assignment=True,
        extra='forbid')

    test_score: int = pydantic.Field(
        description="CT result [0-30]",
        ge=0, le=30)
    fracture_character: typing.Optional[typing.Literal[
        'SDN', 'SP', 'SC', 'RES', 'RP', 'PC', 'BRK',
        'Clean', 'Rough', 'Irregular',
        'Q1', 'Q2', 'Q3']] = pydantic.Field(
            None,
            description="Fracture characteristic among: \n\n"
            "- SP: Sudden planar\n"
        "- SC: Sudden collapse\n"
        "- SDN: Sudden (both)\n"
        "- RP: Resistant planar\n"
        "- PC: Progressive compression\n"
        "- RES: Resistant (both)\n"
        "- BRK or B: Break\n"
        "- Clean\n"
        "- Rough\n"
        "- Irregular\n"
        "- Q1\n"
        "- Q2\n"
        "- Q3")


class CTStabilityTest(_StabilityTest):
    """
    Class for CT (Compression test) stability test.
    """
    results: typing.List[CTStabilityTestResult] = pydantic.Field(
        [],
        description="Successive results of a single CT test. No results mean CT31.")
    type: typing.Literal['CT'] = 'CT'


class ECTStabilityTestResult(_StabilityTestResult):
    """
    Class for ECT (extended column test) stability test result.

    Set score=0 for ECTPV.
    """
    test_score: int = pydantic.Field(
        description="CT result [0-30]",
        ge=0, le=30)
    propagation: bool = False


class ECTStabilityTest(_StabilityTest):
    """
    Class for ECT (Extended column test) stability test.
    """
    results: typing.List[ECTStabilityTestResult] = pydantic.Field(
        [],
        description="Successive results of a single ECT test. No results mean ECTN.")
    type: typing.Literal['ECT'] = 'ECT'


class PSTStabilityTest(_StabilityTest, _StabilityTestResult):
    """
    Class for PST (Propagation Saw test) stability test.
    """
    column_length: typing.Optional[float] = pydantic.Field(
        None,
        description="Length of the column used for stability test (m).")
    cut_length: float = pydantic.Field(
        description="The cut length necessary to start the propagation.")
    propagation: typing.Literal['End', 'SF', 'Arr'] = pydantic.Field(
        description="Propagation type: \n\n"
        "- End means the fracture propagate through the end of the column,\n"
        "- SF means slab fracture,\n"
        "- Arr means arrest of the propagation before the end of the column.")
    type: typing.Literal['PST'] = 'PST'


class ShearFrameStabilityTestResult(_StabilityTestResult):
    """
    Class for shear frame stability test results.
    """
    model_config = pydantic.ConfigDict(
        validate_assignment=True,
        extra='forbid')

    force: float = pydantic.Field(
        description="Failure force (N).",
        ge=0)
    fracture_character: typing.Optional[typing.Literal[
        'SDN', 'SP', 'SC', 'RES', 'RP', 'PC', 'BRK',
        'Clean', 'Rough', 'Irregular',
        'Q1', 'Q2', 'Q3']] = pydantic.Field(
            None,
            description="Fracture characteristic among: \n\n"
            "- SP: Sudden planar\n"
        "- SC: Sudden collapse\n"
        "- SDN: Sudden (both)\n"
        "- RP: Resistant planar\n"
        "- PC: Progressive compression\n"
        "- RES: Resistant (both)\n"
        "- BRK or B: Break\n"
        "- Clean\n"
        "- Rough\n"
        "- Irregular\n"
        "- Q1\n"
        "- Q2\n"
        "- Q3")


class ShearFrameStabilityTest(_StabilityTest):
    """
    Class for Shear frame stability test.
    """
    results: typing.List[ShearFrameStabilityTestResult] = pydantic.Field(
        [],
        description="Successive results of a single SF test. No results mean no failure.")
    type: typing.Literal['Shear Frame'] = 'Shear Frame'



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import unittest

import snowprofile


class TestSnowProfile(unittest.TestCase):
    def test_init_void_snowprofile(self):
        sp = snowprofile.SnowProfile()
        assert sp.id is None

    def test_example_snowprofile(self):
        sp = snowprofile.SnowProfile()
        sp.location.name = 'Col de Porte'
        sp.location.latitude = 45.295043
        sp.location.longitude = 5.76559
        import datetime
        sp.time.record_time = datetime.datetime(2019, 12, 25, 10, 0)
        sp.time.report_time = datetime.datetime.now()

    def test_add_profile(self):
        sp = snowprofile.SnowProfile()
        dp = snowprofile.profiles.DensityProfile(
            method_of_measurement="Snow Cutter",
            quality_of_measurement="Good",
            probed_thickness=0.03,  # 3cm cutter thickness
            data = {'top_height': [1.2, 1.1, 1.0, 0.5],
                    'thickness': [0.1, 0.1, 0.5, 0.5],
                    'density': [75, 100, 180, 230]}
        )
        sp.density_profiles.append(dp)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import os
import os.path
import unittest

import snowprofile

_here = os.path.dirname(os.path.realpath(__file__))


class TestIOCAAML5XML(unittest.TestCase):

    def test_read_caaml5_xml_default_example(self):
        """
        Test a caaml.org CAAML 5 XML example file
        """
        sp = snowprofile.io.read_caaml5_xml(os.path.join(_here, 'resources', 'Snowprofile_IACS_SLF7245.xml'))

        assert sp.observer.source_id == "SLF"
        assert sp.weather.cloudiness == 'BKN'
        assert len(sp.stability_tests) == 2

    def test_read_caaml5_xml_niviz_example(self):
        """
        Test a random CAAML 5 XML file written by NiViz
        """
        sp = snowprofile.io.read_caaml5_xml(os.path.join(_here, 'resources', 'Profilev5-1.caaml'))

        assert sp.location.name == 'Col de Porte'
        assert sp.weather.cloudiness == 'CLR'
        assert sp.profile_depth == 1.27
        assert len(sp.temperature_profiles) == 1
        assert len(sp.density_profiles) == 1


if __name__ == "__main__":
    unittest.main()
